"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const util_1 = __importDefault(require("util"));
const inquirer_1 = __importDefault(require("inquirer"));
const get_ips_1 = require("./get-ips");
const api_1 = require("../sdk/studio/api");
const PREFIX = '\x1b[34m[CFG]\x1b[0m';
class Config {
    /**
     * Whether a file exists (Node.js API cannot be converted using promisify)
     * @param path
     */
    static async exists(path) {
        let exists = false;
        try {
            await util_1.default.promisify(fs_1.default.stat)(path);
            exists = true;
        }
        catch (ex) {
            /* noop */
        }
        return exists;
    }
    constructor() {
        this._configured = false;
        this._filename = path_1.default.join(os_1.default.homedir(), 'edge-impulse-config.json');
    }
    /**
     * Clean the configuration
     */
    async clean() {
        if (await Config.exists(this._filename)) {
            await util_1.default.promisify(fs_1.default.unlink)(this._filename);
        }
    }
    async removeProjectReferences() {
        let config = await this.load();
        delete config.apiKey;
        delete config.linuxProjectId;
        delete config.uploaderProjectId;
        await this.store(config);
    }
    async getUploaderProjectId() {
        let config = await this.load();
        if (!config) {
            return undefined;
        }
        return config.uploaderProjectId;
    }
    async setUploaderProjectId(projectId) {
        let config = await this.load();
        config.uploaderProjectId = projectId;
        await this.store(config);
    }
    async getLinuxProjectId() {
        let config = await this.load();
        if (!config) {
            return undefined;
        }
        return config.linuxProjectId;
    }
    async setLinuxProjectId(projectId) {
        let config = await this.load();
        config.linuxProjectId = projectId;
        await this.store(config);
    }
    async getLastVersionCheck() {
        let config = await this.load();
        if (!config) {
            return Date.now(); // just installed, it's fine
        }
        return config.lastVersionCheck || Date.now();
    }
    async setLastVersionCheck() {
        let config = await this.load();
        config.lastVersionCheck = Date.now();
        await this.store(config);
    }
    /**
     * Verify that a user has configured the host and was logged in
     */
    async verifyLogin(devMode, apiKey, verifyType = 'project') {
        let config = await this.load();
        if (process.env.EI_HOST && config.host !== process.env.EI_HOST) {
            config.host = process.env.EI_HOST;
            config.jwtToken = '';
        }
        let setDeviceUpload = true;
        // no host? ask the user where to connect
        if (!config.host) {
            if (devMode) {
                let hostRes = await inquirer_1.default.prompt([{
                        type: 'list',
                        choices: [
                            { name: 'edgeimpulse.com (Production)', value: 'edgeimpulse.com' },
                            { name: 'acc1.edgeimpulse.com (Acceptance 1)', value: 'acc1.edgeimpulse.com' },
                            { name: 'acc2.edgeimpulse.com (Acceptance 2)', value: 'acc2.edgeimpulse.com' },
                            { name: 'localhost', value: 'localhost' },
                            { name: 'edgeimpulse.test', value: 'edgeimpulse.test' },
                        ],
                        name: 'host',
                        message: 'To which server do you want to connect? ' +
                            '(you can override this by setting EI_HOST environmental variable)',
                        pageSize: 20
                    }]);
                config.host = hostRes.host;
                config.jwtToken = '';
            }
            else {
                config.host = 'edgeimpulse.com';
                config.jwtToken = '';
            }
        }
        let host = config.host;
        let isLocalhost = host === 'localhost';
        if (host === 'localhost' && get_ips_1.ips.length === 0) {
            console.warn('Cannot set device upload host and ingestion settings, ' +
                'host is set to localhost and no public IP available. ' +
                'Please set EI_HOST environmental variable.');
            setDeviceUpload = false;
        }
        else if (host === 'localhost') {
            host = get_ips_1.ips[0].address;
        }
        const hostIsIP = host === 'localhost' ||
            host === 'host.docker.internal' ||
            /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/.test(host);
        const hostIsHttps = !hostIsIP;
        const wsProtocol = hostIsHttps ? 'wss' : 'ws';
        const httpProtocol = hostIsHttps ? 'https' : 'http';
        // so... we have internal endpoints (used by the daemon) and external endpoints (used by the device)
        // the device does not have TLS configured right now... So we need to always use the HTTP/WS endpoints there...
        const wsEndpointExternal = hostIsIP ? `ws://${host}:4802` : `ws://remote-mgmt.${host}`;
        const ingestionEndpointExternal = hostIsIP ? `http://${host}:4810` : `http://ingestion.${host}`;
        // api endpoint is not called from device, so use HTTPS there
        let apiEndpointExternal = hostIsIP ? `${httpProtocol}://${host}:4800` : `${httpProtocol}://studio.${host}`;
        let apiWspointExternal = hostIsIP ? `${wsProtocol}://${host}:4800` : `${wsProtocol}://studio.${host}`;
        // internal endpoints
        let apiEndpointInternal = isLocalhost ? 'http://localhost:4800' : apiEndpointExternal;
        let apiWsEndpointInternal = isLocalhost ? 'ws://localhost:4800' : apiWspointExternal;
        const wsEndpointInternal = isLocalhost
            ? 'ws://localhost:4802'
            : (hostIsIP ? `${wsProtocol}://${host}:4802` : `${wsProtocol}://remote-mgmt.${host}`);
        const ingestionEndpointInternal = isLocalhost ?
            'http://localhost:4810'
            : hostIsIP ? `${httpProtocol}://${host}:4810` : `${httpProtocol}://ingestion.${host}`;
        this._api = new api_1.EdgeImpulseApi({
            endpoint: apiEndpointInternal,
            extraHeaders: { 'User-Agent': 'EDGE_IMPULSE_CLI' },
        });
        this._endpoints = {
            device: {
                ws: wsEndpointExternal,
                api: apiEndpointExternal,
                ingestion: ingestionEndpointExternal
            },
            internal: {
                ws: wsEndpointInternal,
                api: apiEndpointInternal,
                apiWs: apiWsEndpointInternal,
                ingestion: ingestionEndpointInternal
            }
        };
        if (apiKey) {
            // try and authenticate...
            await this._api.authenticate({
                method: 'apiKey',
                apiKey: apiKey
            });
            config.apiKey = apiKey;
        }
        else {
            if (!config.jwtToken) {
                config.jwtToken = await this.getJWTToken(this._api, config, host);
            }
            await this._api.authenticate({
                method: 'jwtToken',
                jwtToken: config.jwtToken,
            });
        }
        await this.verifyAccess(config, verifyType);
        // fetch user...
        if (config.jwtToken) {
            let user = await this._api.user.getCurrentUser();
            // check if has developer profile...
            if (!user.organizations.find(x => x.isDeveloperProfile)) {
                console.log(PREFIX, 'Creating developer profile...');
                await this._api.user.createDeveloperProfile();
                console.log(PREFIX, 'Creating developer profile OK');
            }
        }
        // OK, now we're OK!
        await this.store(config);
        this._configured = true;
        return {
            api: this._api,
            endpoints: this._endpoints,
            setDeviceUpload: setDeviceUpload,
            host: host,
        };
    }
    async getDataForwarderDevice(deviceId) {
        let config = await this.load();
        return config.dataForwarderDevices[deviceId];
    }
    async storeDataForwarderDevice(deviceId, data) {
        let config = await this.load();
        config.dataForwarderDevices[deviceId] = data;
        await this.store(config);
    }
    async deleteDataForwarderDevice(deviceId) {
        let config = await this.load();
        delete config.dataForwarderDevices[deviceId];
        await this.store(config);
    }
    async getDaemonDevice(deviceId) {
        let config = await this.load();
        return config.daemonDevices[deviceId];
    }
    async storeDaemonDevice(deviceId, data) {
        let config = await this.load();
        config.daemonDevices[deviceId] = data;
        await this.store(config);
    }
    async deleteDaemonDevice(deviceId) {
        let config = await this.load();
        delete config.daemonDevices[deviceId];
        await this.store(config);
    }
    async getCamera() {
        let config = await this.load();
        return config.camera;
    }
    async storeCamera(camera) {
        let config = await this.load();
        config.camera = camera;
        await this.store(config);
    }
    async getAudio() {
        let config = await this.load();
        return config.audio;
    }
    async storeAudio(audio) {
        let config = await this.load();
        config.audio = audio;
        await this.store(config);
    }
    async getRunner() {
        let config = await this.load();
        return config.runner;
    }
    async storeStoragePath(path) {
        let config = await this.load();
        config.runner.storagePath = path;
        await this.store(config);
    }
    async getStoragePath() {
        let config = await this.load();
        return config.runner.storagePath || this.getDefaultStoragePath();
    }
    async getStorageIndex() {
        let config = await this.load();
        return config.runner.storageIndex || 0;
    }
    async storeStorageIndex(index) {
        let config = await this.load();
        config.runner.storageIndex = index;
        await this.store(config);
    }
    async storeProjectId(projectId) {
        let config = await this.load();
        config.runner.projectId = projectId;
        await this.store(config);
    }
    async storeBlockId(blockId) {
        let config = await this.load();
        config.runner.blockId = blockId;
        await this.store(config);
    }
    async setRunnerImpulseIdForProjectId(projectId, impulseId) {
        let config = await this.load();
        config.runner.impulseIdsForProjectId = config.runner.impulseIdsForProjectId || {};
        config.runner.impulseIdsForProjectId[projectId.toString()] = { impulseId };
        await this.store(config);
    }
    async getRunnerImpulseIdForProjectId(projectId) {
        let config = await this.load();
        if (!config.runner.impulseIdsForProjectId) {
            return null;
        }
        let ret = config.runner.impulseIdsForProjectId[projectId.toString()];
        return ret ? ret.impulseId : null;
    }
    async getStudioUrl(whitelabelId) {
        var _a, _b, _c;
        if (whitelabelId !== null) {
            const whitelabelRequest = await ((_a = this._api) === null || _a === void 0 ? void 0 : _a.whitelabels.getWhitelabelDomain(whitelabelId));
            if (whitelabelRequest && whitelabelRequest.success && whitelabelRequest.domain) {
                const protocol = ((_b = this._endpoints) === null || _b === void 0 ? void 0 : _b.internal.api.startsWith('https')) ? 'https' : 'http';
                return `${protocol}://${whitelabelRequest.domain}`;
            }
        }
        return (_c = this._endpoints) === null || _c === void 0 ? void 0 : _c.internal.api.replace('/v1', '');
    }
    getDefaultModelsPath() {
        return path_1.default.join(this.getDefaultRunnerPath(), 'models');
    }
    getDefaultStoragePath() {
        return path_1.default.join(this.getDefaultRunnerPath(), 'storage');
    }
    getDefaultRunnerPath() {
        return path_1.default.join(os_1.default.homedir(), '.ei-linux-runner');
    }
    async load() {
        if (!await Config.exists(this._filename)) {
            return {
                host: '',
                jwtToken: '',
                uploaderProjectId: undefined,
                lastVersionCheck: Date.now(),
                apiKey: undefined,
                dataForwarderDevices: {},
                daemonDevices: {},
                camera: undefined,
                audio: undefined,
                linuxProjectId: undefined,
                runner: {
                    projectId: undefined,
                    blockId: undefined,
                    storageIndex: undefined,
                    storagePath: this.getDefaultStoragePath(),
                    impulseIdsForProjectId: undefined,
                }
            };
        }
        try {
            let c = JSON.parse(await util_1.default.promisify(fs_1.default.readFile)(this._filename, 'utf-8'));
            if (!c.dataForwarderDevices) {
                c.dataForwarderDevices = {};
            }
            if (!c.daemonDevices) {
                c.daemonDevices = {};
            }
            // in v1.6.2 we did not properly filter the sensors array, so filter it out here
            for (let [k, d] of Object.entries(c.dataForwarderDevices)) {
                d.sensors = d.sensors.filter(f => !!f);
            }
            return c;
        }
        catch (ex2) {
            let ex = ex2;
            throw new Error('Failed to parse config ' + this._filename + ' ' + (ex.message || ex));
        }
    }
    /**
     * This'll throw when access failed
     * @param config
     * @param verifyType
     */
    async verifyAccess(config, verifyType) {
        const api = this._api;
        if (!api) {
            throw new Error('api is null');
        }
        if (verifyType === 'project') {
            try {
                await api.projects.listProjects();
            }
            catch (ex2) {
                let ex = ex2;
                if ((ex.message || ex.toString()).indexOf('Invalid API key') > -1) {
                    await api.organizations.listOrganizations();
                    throw new Error(`The API key you provided (${(config.apiKey || '').slice(0, 10)}...) ` +
                        `is an organization API Key. This CLI command requires a project API Key ` +
                        `(or omit the --api-key argument to log in using a username and password).`);
                }
                else {
                    throw ex;
                }
            }
        }
        else {
            try {
                await api.organizations.listOrganizations();
            }
            catch (ex2) {
                let ex = ex2;
                if ((ex.message || ex.toString()).indexOf('Invalid API key') > -1) {
                    await api.projects.listProjects();
                    throw new Error(`The API key you provided (${(config.apiKey || '').slice(0, 10)}...) ` +
                        `is a project API Key. This CLI command requires an organization API Key ` +
                        `(or omit the --api-key argument to log in using a username and password).`);
                }
                else {
                    throw ex;
                }
            }
        }
    }
    async getJWTToken(api, config, host) {
        const username = await inquirer_1.default.prompt({
            type: 'input',
            name: 'username',
            message: `What is your user name or e-mail address (${host})?`
        });
        const { needPassword, email, whitelabels } = await api.user.getUserNeedToSetPassword(username.username);
        if (needPassword) {
            const protocol = `http${config.host === 'localhost' ? '' : 's'}://`;
            const port = `${config.host === 'localhost' ? ':4800' : ''}`;
            const encodedEmail = encodeURIComponent(`${email}`);
            let resetUrl;
            if (whitelabels && whitelabels.length === 1) {
                resetUrl =
                    `${protocol}${whitelabels[0] || config.host}${port}/set-password?email=${encodedEmail}`;
            }
            // Some users may be part of different white labels. In these cases, we cannot provide an
            // URL for them to set the password, so we invite them to visit their user profile.
            let errorMsg = 'To use the CLI you will need to set a password. ';
            errorMsg +=
                resetUrl ?
                    `Go to ${resetUrl} to set one.` :
                    'Go to your user profile settings in Studio to set one.';
            throw new Error(errorMsg);
        }
        const password = await inquirer_1.default.prompt({
            type: 'password',
            name: 'password',
            message: `What is your password?`
        });
        let res;
        try {
            res = (await api.login.login({
                username: username.username,
                password: password.password,
            }));
        }
        catch (ex2) {
            let ex = ex2;
            if ((ex.message || ex.toString()).startsWith('ERR_TOTP_TOKEN IS REQUIRED')) {
                // For TOTP allow the user to enter another one if it's invalid (so run in a loop)
                while (1) {
                    try {
                        const totpToken = await inquirer_1.default.prompt({
                            type: 'input',
                            name: 'totpToken',
                            message: `Enter a code from your authenticator app`
                        });
                        res = (await api.login.login({
                            username: username.username,
                            password: password.password,
                            totpToken: totpToken.totpToken,
                        }));
                        break;
                    }
                    catch (ex3) {
                        let totpEx = ex3;
                        console.warn('Failed to log in:', totpEx.message || totpEx.toString());
                    }
                }
            }
            else {
                throw ex;
            }
        }
        if (!res.token) {
            throw new Error('Authentication did not return a token');
        }
        return res.token;
    }
    async store(config) {
        await util_1.default.promisify(fs_1.default.writeFile)(this._filename, JSON.stringify(config, null, 4), 'utf-8');
    }
}
exports.Config = Config;
//# sourceMappingURL=config.js.map