"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupCliApp = exports.initCliApp = exports.getCliVersion = void 0;
const config_1 = require("./config");
const check_new_version_1 = __importDefault(require("./check-new-version"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const aws_sm_utils_1 = require("./aws-sm-utils");
const aws_iotcore_connector_1 = require("./aws-iotcore-connector");
const version = JSON.parse(fs_1.default.readFileSync(path_1.default.join(__dirname, '..', '..', 'package.json'), 'utf-8')).version;
function getCliVersion() {
    return version;
}
exports.getCliVersion = getCliVersion;
async function initCliApp(opts) {
    if (!opts.silentArgv) {
        console.log(opts.appName + ' v' + version);
    }
    const configFactory = new config_1.Config();
    let config;
    // AWS Support
    let awsSM;
    let awsIOT;
    try {
        if (opts.cleanArgv) {
            await configFactory.clean();
        }
        // AWS: Secrets Manager + IoTCore Connect Integration
        if (opts.greengrassArgv) {
            if (awsSM === undefined) {
                awsSM = await new aws_sm_utils_1.AWSSecretsManagerUtils(opts);
            }
            let smApiKey;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            smApiKey = await awsSM.getEdgeImpulseAPIKeyFromSecretsManager();
            if (smApiKey !== undefined && smApiKey.length > 0) {
                // assign if we capture the API key via SM...
                opts.apiKeyArgv = smApiKey;
                // we only need AWS IoTCore connectivity for the runner app...
                if (opts.appName === "Edge Impulse Linux runner") {
                    // success - allocate the AWS IoT Helper
                    awsIOT = new aws_iotcore_connector_1.AWSIoTCoreConnector(opts);
                    if (awsIOT !== undefined) {
                        if (!opts.silentArgv) {
                            console.log(opts.appName + ": Connecting to IoTCore...");
                        }
                        const connected = await awsIOT.connect();
                        if (connected) {
                            // success
                            console.log(opts.appName + ": Connected to IoTCore Successfully!");
                            // since we are not running a model, we have to nail/fudge our model info
                            await awsIOT.initModelInfo(opts.appName.replace(" ", "_"), "v" + version, {});
                            // launch all async tasks
                            console.log(opts.appName + ": launching async tasks...");
                            // eslint-disable-next-line @typescript-eslint/no-floating-promises
                            awsIOT.launchAsyncTasks();
                        }
                        else {
                            // failure
                            console.log(opts.appName + ": FAILED to connect to IoTCore");
                        }
                    }
                }
                // continue...
                await configFactory.removeProjectReferences();
            }
            else {
                if (!opts.silentArgv) {
                    // unable to continue as no API key was found in Secrets Manager
                    console.log(opts.appName + " ERROR: Unable to find EI API Key within AWS Secrets Manager. Check AWS configuration. Continuing...");
                }
            }
        }
        // AWS: Secrets Manager + IoTCore Connect Integration
        if (opts.apiKeyArgv) {
            await configFactory.removeProjectReferences();
        }
        try {
            await (0, check_new_version_1.default)(configFactory);
        }
        catch (ex) {
            /* noop */
        }
        // this verifies host settings and verifies the JWT token
        try {
            config = await configFactory.verifyLogin(opts.devArgv, opts.apiKeyArgv);
        }
        catch (ex2) {
            let ex = ex2;
            let msg = ex.message || ex.toString();
            if (msg.indexOf('need to set a password') > -1) {
                console.log('');
                console.log('\x1b[33mWARN\x1b[0m', ex.message);
                console.log('');
                process.exit(1);
            }
            else if (msg.indexOf('Password is incorrect') > -1 ||
                msg.indexOf('User not found') > -1) {
                console.log('');
                console.log('\x1b[33mWARN\x1b[0m', ex.message);
                console.log('');
            }
            else if (msg.indexOf('The API key you provided') > -1) {
                // immediately jump to the "Failed to authenticate" printing
                throw ex;
            }
            else {
                console.log('Stored token seems invalid, clearing cache...');
                console.log(ex.message);
            }
            await configFactory.clean();
            config = await configFactory.verifyLogin(opts.devArgv, opts.apiKeyArgv);
        }
    }
    catch (ex2) {
        let ex = ex2;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        if (ex.statusCode) {
            console.error('Failed to authenticate with Edge Impulse:', 
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            ex.statusCode, ex.response.body);
        }
        else {
            console.error('Failed to authenticate with Edge Impulse:', ex.message || ex.toString());
        }
        process.exit(1);
    }
    // AWS Integration additions
    if (awsSM !== undefined && awsIOT !== undefined) {
        return {
            configFactory,
            config,
            awsSM,
            awsIOT
        };
    }
    return {
        configFactory,
        config
    };
}
exports.initCliApp = initCliApp;
async function setupCliApp(configFactory, config, opts, deviceId) {
    let projectId = await configFactory.getUploaderProjectId();
    if (projectId) {
        try {
            let projectInfoReq = (await config.api.projects.getProjectInfo(projectId, {}));
            if (!opts.silentArgv) {
                console.log('    Project:    ', projectInfoReq.project.name + ' (ID: ' + projectId + ')');
                console.log('');
            }
        }
        catch (ex2) {
            let ex = ex2;
            console.warn('Cannot read cached project (' + (ex.message || ex.toString()) + ')');
            projectId = undefined;
        }
    }
    if (!projectId) {
        if (!opts.silentArgv) {
            console.log('');
        }
        let fromConfig = opts.getProjectFromConfig ?
            await opts.getProjectFromConfig(deviceId) :
            undefined;
        let projectList = (await config.api.projects.listProjects());
        if (!projectList.projects || projectList.projects.length === 0) {
            console.log('This user has no projects, create one before continuing');
            process.exit(1);
        }
        else if (fromConfig) {
            projectId = fromConfig.projectId;
        }
        else if (projectList.projects && projectList.projects.length === 1) {
            projectId = projectList.projects[0].id;
        }
        else {
            let inqRes = await inquirer_1.default.prompt([{
                    type: 'list',
                    choices: (projectList.projects || []).map(p => ({ name: p.owner + ' / ' + p.name, value: p.id })),
                    name: 'project',
                    message: opts.connectProjectMsg,
                    pageSize: 20
                }]);
            projectId = Number(inqRes.project);
        }
        if (!projectId) {
            throw new Error('projectId is null');
        }
    }
    let devKeys = {
        apiKey: opts.apiKeyArgv || '',
        hmacKey: opts.hmacKeyArgv || '0'
    };
    if (!opts.apiKeyArgv) {
        try {
            let dk = (await config.api.projects.listDevkeys(projectId));
            if (!dk.apiKey) {
                throw new Error('No API key set (via --api-key), and no development API keys configured for ' +
                    'this project. Add a development API key from the Edge Impulse dashboard to continue.');
            }
            devKeys.apiKey = dk.apiKey;
            if (!opts.hmacKeyArgv && dk.hmacKey) {
                devKeys.hmacKey = dk.hmacKey;
            }
        }
        catch (ex2) {
            let ex = ex2;
            throw new Error('Failed to load development keys: ' + (ex.message || ex.toString()));
        }
    }
    return {
        projectId,
        devKeys,
    };
}
exports.setupCliApp = setupCliApp;
//# sourceMappingURL=init-cli-app.js.map