"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockConfigManager = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const blocks_helper_1 = require("./blocks-helper");
const inquirer_1 = __importDefault(require("inquirer"));
class BlockConfigManager {
    constructor(config, folder, opts) {
        this._config = config;
        this._folder = folder;
        this._paths = {
            eiBlockConfig: path_1.default.join(folder, '.ei-block-config'),
            parametersJson: path_1.default.join(folder, 'parameters.json'),
        };
        this._skipConfirmation = (opts === null || opts === void 0 ? void 0 : opts.skipConfirmation) || false;
        this._transformWhatTypeOfBlockIsThisReply = opts === null || opts === void 0 ? void 0 : opts.transformWhatTypeOfBlockIsThisReply;
    }
    async loadConfig(opts) {
        try {
            let loadedConfig = await (0, blocks_helper_1.pathExists)(this._paths.eiBlockConfig) ?
                JSON.parse(await fs_1.default.promises.readFile(this._paths.eiBlockConfig, 'utf-8')) : undefined;
            if (loadedConfig) {
                if (typeof loadedConfig.version === 'undefined') {
                    loadedConfig = await this.migrateV0ToV2(loadedConfig);
                    await fs_1.default.promises.writeFile(this._paths.eiBlockConfig, JSON.stringify(loadedConfig, null, 4), 'utf-8');
                    if (!this._skipConfirmation) {
                        console.log(`✓ Migrated configuration`);
                        console.log(``);
                    }
                }
                if (loadedConfig.version === 1) {
                    loadedConfig = await this.migrateV1ToV2(loadedConfig);
                    await fs_1.default.promises.writeFile(this._paths.eiBlockConfig, JSON.stringify(loadedConfig, null, 4), 'utf-8');
                    if (!this._skipConfirmation) {
                        console.log(`✓ Migrated configuration`);
                        console.log(``);
                    }
                }
                if (loadedConfig.version !== 2) {
                    throw new Error(`Config version is ${loadedConfig.version} (expected 1 or 2). ` +
                        `Delete the .ei-block-config file to reset the block.`);
                }
            }
            let config;
            if (opts.clean) {
                config = null;
            }
            else {
                config = loadedConfig ?
                    loadedConfig.config[this._config.host] || null :
                    null;
            }
            if (!await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                if (opts.throwOnMissingParams) {
                    throw new Error(`Missing "parameters.json", one is required for all blocks`);
                }
                return null;
            }
            let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (!parameters.type) {
                if (parameters.info) {
                    // DSP Block
                    parameters = {
                        version: 1,
                        type: 'dsp',
                        info: parameters.info,
                        parameters: parameters.parameters,
                    };
                    await this.saveParameters(parameters);
                }
                else if (Array.isArray(parameters)) {
                    return null;
                }
            }
            switch (parameters.type) {
                case 'dsp': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'dsp',
                        config: config,
                        parameters: parameters,
                    };
                }
                case 'deploy': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'deploy',
                        config: config,
                        parameters: parameters,
                    };
                }
                case 'machine-learning': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'machine-learning',
                        config: config,
                        parameters: parameters,
                    };
                }
                case 'transform': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'transform',
                        config: config,
                        parameters: parameters,
                    };
                }
                case 'synthetic-data': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'synthetic-data',
                        config: config,
                        parameters: parameters,
                    };
                }
                case 'ai-action': {
                    if (parameters.version !== 1) {
                        throw new Error(`Unexpected value for "version" in "parameters.json" (expected 1)`);
                    }
                    return {
                        type: 'ai-action',
                        config: config,
                        parameters: parameters,
                    };
                }
                default:
                    throw new Error(`Unhandled value for "type" in "parameters.json" (${parameters.type})`);
            }
        }
        catch (ex2) {
            let ex = ex2;
            throw new Error('Failed to load config from .ei-block-config: ' +
                (ex.message || ex.toString()));
        }
    }
    async saveConfig(config) {
        let currConfig;
        if (await (0, blocks_helper_1.pathExists)(this._paths.eiBlockConfig)) {
            try {
                let loadedConfig = JSON.parse(await fs_1.default.promises.readFile(this._paths.eiBlockConfig, 'utf-8'));
                if (loadedConfig.version !== 2) {
                    throw new Error(`Config version is ${loadedConfig.version} (expected 2). ` +
                        'This file should have been auto-converted already when calling "BlockConfigManager:saveConfig".');
                }
                currConfig = loadedConfig;
            }
            catch (ex2) {
                let ex = ex2;
                throw new Error('Failed to load config from .ei-block-config: ' +
                    (ex.message || ex.toString()));
            }
        }
        else {
            currConfig = {
                version: 2,
                config: {},
            };
        }
        currConfig.config[this._config.host] = config;
        await fs_1.default.promises.writeFile(this._paths.eiBlockConfig, JSON.stringify(currConfig, null, 4), 'utf-8');
    }
    async saveParameters(parameters) {
        await fs_1.default.promises.writeFile(this._paths.parametersJson, JSON.stringify(parameters, null, 4), 'utf-8');
    }
    /**
     * Migrate v0 config to v1 to v2, see top of this file for an explanation.
     */
    async migrateV0ToV2(config) {
        let v1Config = {
            version: 1,
            config: {},
        };
        v1Config.config[this._config.host] = config;
        return await this.migrateV1ToV2(v1Config);
    }
    /**
     * Migrate v1 to v2, see top of this file for an explanation.
     */
    async migrateV1ToV2(config) {
        if (!this._skipConfirmation) {
            const continueRes = await inquirer_1.default.prompt({
                type: 'confirm',
                name: 'continue',
                message: `We're moving some of your block configuration from .ei-block-config to parameters.json. ` +
                    `Continuing will update both files. Continue?`
            });
            if (!continueRes.continue) {
                throw new Error(`Aborting`);
            }
        }
        let overwroteHost = false;
        // do we have an item for this host? use that. otherwise pick the first one.
        let blockConfigItem;
        if (config.config[this._config.host]) {
            blockConfigItem = config.config[this._config.host];
        }
        else if (Object.keys(config.config).length > 0) {
            blockConfigItem = config.config[Object.keys(config.config)[0]];
            overwroteHost = true;
        }
        else {
            // no items to be found... so just create the new config
            const newConfig = {
                version: 2,
                config: {}
            };
            return newConfig;
        }
        if (blockConfigItem.type === 'dsp') {
            // We're not fetching the remote DSP block here (like we do for all other block types)
            // because there are no properties that are missing in v1 compared to v2
            // (so we can get all of them from the .ei-block-config)
            if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                if (parameters.version !== 1 || parameters.type !== 'dsp') {
                    parameters = {
                        version: 1,
                        type: 'dsp',
                        info: parameters.info,
                        parameters: parameters.parameters,
                    };
                }
                if (parameters.info && typeof parameters.info.port === 'undefined') {
                    parameters.info.port = blockConfigItem.port;
                }
                await this.saveParameters(parameters);
            }
            else {
                throw new Error(`You're missing a "parameters.json" file, this is required for DSP blocks. ` +
                    `Add it and re-run this command.`);
            }
            const newConfig = {
                version: 2,
                config: {}
            };
            for (let k of Object.keys(config.config)) {
                const oldConfig = config.config[k];
                newConfig.config[k] = {
                    organizationId: oldConfig.organizationId,
                    id: oldConfig.id,
                };
                if (typeof newConfig.config[k].id === 'undefined') {
                    delete newConfig.config[k].id;
                }
            }
            return newConfig;
        }
        else if (blockConfigItem.type === 'deploy') {
            if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                if (parameters.version !== 1 || parameters.type !== 'deploy') {
                    parameters = {
                        version: 1,
                        type: 'deploy',
                        info: parameters.info || {},
                    };
                }
                parameters.info = parameters.info || {};
                if (typeof parameters.info.name === 'undefined') {
                    parameters.info.name = blockConfigItem.name;
                }
                if (typeof parameters.info.description === 'undefined') {
                    parameters.info.description = blockConfigItem.description;
                }
                if (typeof parameters.info.category === 'undefined') {
                    parameters.info.category = blockConfigItem.deployCategory;
                }
                await this.saveParameters(parameters);
            }
            else {
                let parameters;
                if (blockConfigItem.id && !overwroteHost) {
                    try {
                        // the v1 config format was missing a ton of properties that could be configured
                        // on the deploy block. Rather than prompt the user for them we'll fetch the
                        // remote block and copy the properties from EI => parameters.json, so you can
                        // commit the full configured block.
                        const block = await this._config.api.organizationBlocks.getOrganizationDeployBlock(blockConfigItem.organizationId, blockConfigItem.id);
                        parameters = {
                            version: 1,
                            type: 'deploy',
                            info: {
                                name: blockConfigItem.name || block.deployBlock.name,
                                description: blockConfigItem.description || block.deployBlock.description,
                                category: blockConfigItem.deployCategory,
                                cliArguments: block.deployBlock.cliArguments,
                                mountLearnBlock: block.deployBlock.mountLearnBlock,
                                showOptimizations: block.deployBlock.showOptimizations,
                                supportsEonCompiler: block.deployBlock.supportsEonCompiler,
                                integrateUrl: block.deployBlock.integrateUrl,
                                privileged: block.deployBlock.privileged,
                            },
                        };
                    }
                    catch (ex2) {
                        let ex = ex2;
                        throw new Error(`Failed to get deploy block with ID ${blockConfigItem.id}: ` +
                            `${ex.message || ex.toString()}. Does this block still exist? If not, re-init the block ` +
                            `via "edge-impulse-blocks init --clean"`);
                    }
                }
                else {
                    parameters = {
                        version: 1,
                        type: 'deploy',
                        info: {
                            name: blockConfigItem.name,
                            description: blockConfigItem.description,
                            category: blockConfigItem.deployCategory || 'firmware',
                            cliArguments: '',
                            mountLearnBlock: false,
                            showOptimizations: true,
                            supportsEonCompiler: true,
                        },
                    };
                }
                await this.saveParameters(parameters);
            }
            const newConfig = {
                version: 2,
                config: {}
            };
            for (let k of Object.keys(config.config)) {
                const oldConfig = config.config[k];
                newConfig.config[k] = {
                    organizationId: oldConfig.organizationId,
                    id: oldConfig.id,
                };
                if (typeof newConfig.config[k].id === 'undefined') {
                    delete newConfig.config[k].id;
                }
            }
            return newConfig;
        }
        else if (blockConfigItem.type === 'transferLearning') {
            let currBlock;
            if (blockConfigItem.id && !overwroteHost) {
                try {
                    // the v1 config format was missing a few properties that could be configured
                    // on the ML block. Rather than prompt the user for them we'll fetch the
                    // remote block and copy the properties from EI => parameters.json, so you can
                    // commit the full configured block.
                    currBlock = (await this._config.api.organizationBlocks.getOrganizationTransferLearningBlock(blockConfigItem.organizationId, blockConfigItem.id)).transferLearningBlock;
                }
                catch (ex2) {
                    let ex = ex2;
                    throw new Error(`Failed to get ML block with ID ${blockConfigItem.id}: ` +
                        `${ex.message || ex.toString()}. Does this block still exist? If not, re-init the block ` +
                        `via "edge-impulse-blocks init --clean"`);
                }
            }
            if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                if (Array.isArray(parameters)) {
                    // old param file
                    let oldParams = parameters;
                    parameters = {
                        version: 1,
                        type: 'machine-learning',
                        info: {
                            name: blockConfigItem.name,
                            description: blockConfigItem.description,
                            operatesOn: blockConfigItem.tlOperatesOn || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.operatesOn),
                            objectDetectionLastLayer: blockConfigItem.tlObjectDetectionLastLayer ||
                                (currBlock === null || currBlock === void 0 ? void 0 : currBlock.objectDetectionLastLayer),
                            imageInputScaling: blockConfigItem.tlImageInputScaling || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.imageInputScaling),
                            customModelVariants: blockConfigItem.tlCustomModelVariants ||
                                (currBlock === null || currBlock === void 0 ? void 0 : currBlock.customModelVariants),
                            indRequiresGpu: typeof blockConfigItem.tlIndRequiresGpu === 'boolean' ?
                                blockConfigItem.tlIndRequiresGpu :
                                currBlock === null || currBlock === void 0 ? void 0 : currBlock.indRequiresGpu,
                            repositoryUrl: blockConfigItem.repositoryUrl || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.repositoryUrl),
                            displayCategory: currBlock === null || currBlock === void 0 ? void 0 : currBlock.displayCategory,
                        },
                        parameters: oldParams,
                    };
                    await this.saveParameters(parameters);
                }
                else {
                    // new format... just leave it as-is
                }
            }
            else {
                let parameters = {
                    version: 1,
                    type: 'machine-learning',
                    info: {
                        name: blockConfigItem.name,
                        description: blockConfigItem.description,
                        operatesOn: blockConfigItem.tlOperatesOn || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.operatesOn),
                        objectDetectionLastLayer: blockConfigItem.tlObjectDetectionLastLayer ||
                            (currBlock === null || currBlock === void 0 ? void 0 : currBlock.objectDetectionLastLayer),
                        imageInputScaling: blockConfigItem.tlImageInputScaling || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.imageInputScaling),
                        customModelVariants: blockConfigItem.tlCustomModelVariants ||
                            (currBlock === null || currBlock === void 0 ? void 0 : currBlock.customModelVariants),
                        indRequiresGpu: typeof blockConfigItem.tlIndRequiresGpu === 'boolean' ?
                            blockConfigItem.tlIndRequiresGpu :
                            currBlock === null || currBlock === void 0 ? void 0 : currBlock.indRequiresGpu,
                        repositoryUrl: blockConfigItem.repositoryUrl || (currBlock === null || currBlock === void 0 ? void 0 : currBlock.repositoryUrl),
                        displayCategory: currBlock === null || currBlock === void 0 ? void 0 : currBlock.displayCategory,
                    },
                    parameters: [],
                };
                await this.saveParameters(parameters);
            }
            const newConfig = {
                version: 2,
                config: {}
            };
            for (let k of Object.keys(config.config)) {
                const oldConfig = config.config[k];
                newConfig.config[k] = {
                    organizationId: oldConfig.organizationId,
                    id: oldConfig.id,
                };
                if (typeof newConfig.config[k].id === 'undefined') {
                    delete newConfig.config[k].id;
                }
            }
            return newConfig;
        }
        else if (blockConfigItem.type === 'transform') {
            let blockType;
            let currBlock;
            if (blockConfigItem.id && !overwroteHost) {
                try {
                    // the v1 config format was missing a ton of properties that could be configured
                    // on the transform block. Rather than prompt the user for them we'll fetch the
                    // remote block and copy the properties from EI => parameters.json, so you can
                    // commit the full configured block.
                    currBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(blockConfigItem.organizationId, blockConfigItem.id)).transformationBlock;
                    if (currBlock.showInSyntheticData) {
                        blockType = 'synthetic-data';
                    }
                    else if (currBlock.showInAIActions) {
                        blockType = 'ai-actions';
                    }
                    else {
                        blockType = 'transform';
                    }
                }
                catch (ex2) {
                    let ex = ex2;
                    throw new Error(`Failed to get transformation block with ID ${blockConfigItem.id}: ` +
                        `${ex.message || ex.toString()}. Does this block still exist? If not, re-init the block ` +
                        `via "edge-impulse-blocks init --clean"`);
                }
            }
            else {
                if (this._transformWhatTypeOfBlockIsThisReply) {
                    blockType = this._transformWhatTypeOfBlockIsThisReply;
                }
                else {
                    let blockList = [
                        {
                            name: 'Transformation block',
                            value: 'transform'
                        },
                        {
                            name: 'Synthetic data block',
                            value: 'synthetic-data'
                        },
                        {
                            name: 'AI labeling block',
                            value: 'ai-actions'
                        },
                    ];
                    // Select the type of block
                    let blockTypeInqRes = await inquirer_1.default.prompt([{
                            type: 'list',
                            choices: blockList,
                            name: 'type',
                            message: 'What type of block is this?',
                            pageSize: 20
                        }]);
                    blockType = blockTypeInqRes.type;
                }
            }
            const requiredEnvVariables = ((currBlock === null || currBlock === void 0 ? void 0 : currBlock.environmentVariables) || []).length > 0 ?
                ((currBlock === null || currBlock === void 0 ? void 0 : currBlock.environmentVariables) || []).map(x => x.key) :
                undefined;
            if (blockType === 'synthetic-data') {
                if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                    let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                    if (Array.isArray(parameters)) {
                        // old param file
                        let oldParams = parameters;
                        parameters = {
                            version: 1,
                            type: 'synthetic-data',
                            info: {
                                name: blockConfigItem.name,
                                description: blockConfigItem.description,
                                requiredEnvVariables: requiredEnvVariables,
                            },
                            parameters: oldParams,
                        };
                        await this.saveParameters(parameters);
                    }
                    else {
                        // new format... just leave it as-is
                    }
                }
                else {
                    let parameters = {
                        version: 1,
                        type: 'synthetic-data',
                        info: {
                            name: blockConfigItem.name,
                            description: blockConfigItem.description,
                            requiredEnvVariables: requiredEnvVariables,
                        },
                        parameters: [],
                    };
                    await this.saveParameters(parameters);
                }
            }
            else if (blockType === 'ai-actions') {
                if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                    let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                    if (Array.isArray(parameters)) {
                        // old param file
                        let oldParams = parameters;
                        parameters = {
                            version: 1,
                            type: 'ai-action',
                            info: {
                                name: blockConfigItem.name,
                                description: blockConfigItem.description,
                                requiredEnvVariables: requiredEnvVariables,
                                operatesOn: undefined,
                            },
                            parameters: oldParams,
                        };
                        await this.saveParameters(parameters);
                    }
                    else {
                        // new format... just leave it as-is
                    }
                }
                else {
                    let parameters = {
                        version: 1,
                        type: 'ai-action',
                        info: {
                            name: blockConfigItem.name,
                            description: blockConfigItem.description,
                            requiredEnvVariables: requiredEnvVariables,
                            operatesOn: undefined,
                        },
                        parameters: [],
                    };
                    await this.saveParameters(parameters);
                }
            }
            else {
                if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
                    let parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
                    if (Array.isArray(parameters)) {
                        // old param file
                        let oldParams = parameters;
                        parameters = {
                            version: 1,
                            type: 'transform',
                            info: {
                                name: blockConfigItem.name,
                                description: blockConfigItem.description,
                                operatesOn: blockConfigItem.operatesOn,
                                transformMountpoints: blockConfigItem.transformMountpoints,
                                allowExtraCliArguments: currBlock === null || currBlock === void 0 ? void 0 : currBlock.allowExtraCliArguments,
                                cliArguments: currBlock === null || currBlock === void 0 ? void 0 : currBlock.cliArguments,
                                indMetadata: currBlock === null || currBlock === void 0 ? void 0 : currBlock.indMetadata,
                                showInCreateTransformationJob: currBlock === null || currBlock === void 0 ? void 0 : currBlock.showInCreateTransformationJob,
                                showInDataSources: currBlock === null || currBlock === void 0 ? void 0 : currBlock.showInDataSources,
                                requiredEnvVariables: requiredEnvVariables,
                            },
                            parameters: oldParams,
                        };
                        await this.saveParameters(parameters);
                    }
                }
                else {
                    let parameters = {
                        version: 1,
                        type: 'transform',
                        info: {
                            name: blockConfigItem.name,
                            description: blockConfigItem.description,
                            operatesOn: blockConfigItem.operatesOn,
                            transformMountpoints: blockConfigItem.transformMountpoints,
                            allowExtraCliArguments: currBlock === null || currBlock === void 0 ? void 0 : currBlock.allowExtraCliArguments,
                            cliArguments: currBlock === null || currBlock === void 0 ? void 0 : currBlock.cliArguments,
                            indMetadata: currBlock === null || currBlock === void 0 ? void 0 : currBlock.indMetadata,
                            showInCreateTransformationJob: currBlock === null || currBlock === void 0 ? void 0 : currBlock.showInCreateTransformationJob,
                            showInDataSources: currBlock === null || currBlock === void 0 ? void 0 : currBlock.showInDataSources,
                            requiredEnvVariables: requiredEnvVariables,
                        },
                        parameters: [],
                    };
                    await this.saveParameters(parameters);
                }
            }
            const newConfig = {
                version: 2,
                config: {}
            };
            for (let k of Object.keys(config.config)) {
                const oldConfig = config.config[k];
                newConfig.config[k] = {
                    organizationId: oldConfig.organizationId,
                    id: oldConfig.id,
                };
                if (typeof newConfig.config[k].id === 'undefined') {
                    delete newConfig.config[k].id;
                }
            }
            return newConfig;
        }
        else {
            throw new Error(`BlockConfigManager:migrateV1ToV2 failed to handle "${blockConfigItem.type}"`);
        }
    }
}
exports.BlockConfigManager = BlockConfigManager;
//# sourceMappingURL=block-config-manager.js.map