"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.guessRepoUrl = exports.deepCompare = exports.spinner = exports.pathExists = exports.bytesToSize = exports.sleep = void 0;
const util_1 = __importDefault(require("util"));
const fs_1 = __importDefault(require("fs"));
const spawn_helper_1 = require("../spawn-helper");
function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
exports.sleep = sleep;
function bytesToSize(bytes) {
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
    if (bytes === 0)
        return '0 Bytes';
    let i = Number(Math.floor(Math.log(bytes) / Math.log(1024)));
    return Math.round(bytes / Math.pow(1024, i)) + ' ' + sizes[i];
}
exports.bytesToSize = bytesToSize;
async function pathExists(path) {
    let x = false;
    try {
        await util_1.default.promisify(fs_1.default.stat)(path);
        x = true;
    }
    catch (ex) {
        /* noop */
    }
    return x;
}
exports.pathExists = pathExists;
/**
 * Spinner on the terminal
 * @returns Interval (just call clearInterval to stop the spinner)
 */
function spinner() {
    const spinChars = ['-', '\\', '|', '/'];
    let spinIx = -1;
    return setInterval(() => {
        spinIx++;
        spinIx = spinIx % (spinChars.length);
        process.stdout.write('\b' + (spinChars[spinIx]));
    }, 250);
}
exports.spinner = spinner;
/**
 * Deep compare two objects (underneaths JSON stringifies them)
 */
function deepCompare(obj1, obj2) {
    // keys need to be ordered first
    const orderObject = (unordered) => {
        const ordered = Object.keys(unordered).sort().reduce((curr, key) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            curr[key] = unordered[key];
            return curr;
        }, {});
        for (let k of Object.keys(ordered)) {
            if (Array.isArray(ordered[k])) {
                continue;
            }
            if (ordered[k] instanceof Date) {
                continue;
            }
            if (ordered[k] instanceof Object) {
                ordered[k] = orderObject(ordered[k]);
            }
        }
        return ordered;
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    let obj1Ordered = obj1 instanceof Object ?
        // eslint-disable-next-line
        orderObject(obj1) :
        obj1;
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    let obj2Ordered = obj1 instanceof Object ?
        // eslint-disable-next-line
        orderObject(obj2) :
        obj2;
    return JSON.stringify(obj1Ordered) === JSON.stringify(obj2Ordered);
}
exports.deepCompare = deepCompare;
/**
 * Try and guess the git repository URL (by running `git remote -v`). This is used
 * for blocks with a `repositoryUrl` property.
 * @returns a URL or undefined
 */
async function guessRepoUrl() {
    try {
        let gitOutput = await (0, spawn_helper_1.spawnHelper)('git', ['remote', '-v'], {
            cwd: process.cwd(),
            ignoreErrors: false,
        });
        let line = gitOutput.split('\n').find(x => x.startsWith('origin') && x.indexOf('fetch') > -1);
        if (line) {
            let urlPart = line.split(/\s+/)[1];
            if (urlPart.startsWith('git@')) {
                urlPart = urlPart.replace(':', '/');
                urlPart = urlPart.replace('git@', 'https://');
            }
            if (urlPart.endsWith('.git')) {
                urlPart = urlPart.slice(0, urlPart.length - 4);
            }
            return urlPart;
        }
        return undefined;
    }
    catch (ex) {
        return undefined;
    }
}
exports.guessRepoUrl = guessRepoUrl;
//# sourceMappingURL=blocks-helper.js.map