"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitCLIBlock = void 0;
const inquirer_1 = __importDefault(require("inquirer"));
const path_1 = __importDefault(require("path"));
const blocks_helper_1 = require("./blocks-helper");
const fs_1 = __importDefault(require("fs"));
class InitCLIBlock {
    constructor(config, folder, blockConfigManager) {
        this._config = config;
        this._folder = folder;
        this._paths = {
            eiBlockConfig: path_1.default.join(folder, '.ei-block-config'),
            parametersJson: path_1.default.join(folder, 'parameters.json'),
        };
        this._blockConfigManager = blockConfigManager;
    }
    async getOrganization() {
        const config = this._config;
        // Select the organization
        let organizations = await config.api.organizations.listOrganizations();
        let organizationId;
        if (!organizations.organizations || organizations.organizations.length === 0) {
            throw new Error('User is not part of any Edge Impulse organizations. You can only use custom blocks if ' +
                'you have access to the enterprise version of Edge Impulse. You can log in with a new account via ' +
                '`edge-impulse-blocks --clean`.');
        }
        else if (organizations.organizations && organizations.organizations.length === 1) {
            organizationId = organizations.organizations[0].id;
        }
        else {
            let orgInqRes = await inquirer_1.default.prompt([{
                    type: 'list',
                    choices: (organizations.organizations || []).map(p => {
                        let name = p.name;
                        if (p.isDeveloperProfile) {
                            name += ' (personal account)';
                        }
                        return {
                            name: name,
                            value: p.id,
                        };
                    }),
                    name: 'organization',
                    message: 'In which organization do you want to create this block?',
                    pageSize: 20
                }]);
            organizationId = Number(orgInqRes.organization);
        }
        const organizationInfo = (await config.api.organizations.getOrganizationInfo(organizationId));
        return organizationInfo;
    }
    async initBlock(type, organizationInfo) {
        switch (type) {
            case 'dsp':
                return await this.initDSPBlock(organizationInfo);
            case 'deploy':
                return await this.initDeployBlock(organizationInfo);
            case 'machine-learning':
                return await this.initMachineLearningBlock(organizationInfo);
            case 'synthetic-data':
                return await this.initSyntheticDataBlock(organizationInfo);
            case 'ai-action':
                return await this.initAIActionBlock(organizationInfo);
            case 'transform':
                return await this.initTransformBlock(organizationInfo);
            default:
                throw new Error(`initBlock missing type "${type}"`);
        }
    }
    async initDSPBlock(organizationInfo) {
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let parameters;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            parameters = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (!parameters.version || !parameters.type) {
                parameters = {
                    version: parameters.version || 1,
                    type: parameters.type || 'dsp',
                    info: parameters.info,
                    parameters: parameters.parameters,
                };
                await this._blockConfigManager.saveParameters(parameters);
            }
        }
        let existingBlocks = (await api.organizationBlocks.listOrganizationDspBlocks(organizationId)).dspBlocks;
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        if (!parameters) {
            console.log('');
            console.log(`You'll need a parameters.json file describing this block before you can push it.`);
            console.log(`See https://docs.edgeimpulse.com/docs/edge-impulse-studio/processing-blocks/custom-blocks`);
        }
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async initDeployBlock(organizationInfo) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let existingBlocks = (await api.organizationBlocks.listOrganizationDeployBlocks(organizationId)).deployBlocks;
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        let params;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            params = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (!params.version) {
                params.version = 1;
            }
            params.type = 'deploy';
            params.info = params.info || {};
        }
        let blockName;
        if ((_a = params === null || params === void 0 ? void 0 : params.info) === null || _a === void 0 ? void 0 : _a.name) {
            blockName = params.info.name;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.name) {
            blockName = selectedBlock.name;
        }
        else {
            blockName = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Enter the name of your block (min. 2 characters)',
                }])).name;
            if (blockName.length < 2) {
                throw new Error('New block must have a name longer than 2 characters.');
            }
        }
        let blockDescription;
        if ((_b = params === null || params === void 0 ? void 0 : params.info) === null || _b === void 0 ? void 0 : _b.description) {
            blockDescription = params.info.description;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.description) {
            blockDescription = selectedBlock.description;
        }
        else {
            blockDescription = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'description',
                    message: 'Enter the description of your block',
                }])).description;
        }
        let category;
        if ((_c = params === null || params === void 0 ? void 0 : params.info) === null || _c === void 0 ? void 0 : _c.category) {
            category = params.info.category;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.category) {
            category = selectedBlock.category;
        }
        else {
            category = (await inquirer_1.default.prompt([{
                    type: 'list',
                    name: 'category',
                    choices: [
                        {
                            name: 'Library',
                            value: 'library'
                        },
                        {
                            name: 'Firmware',
                            value: 'firmware'
                        }
                    ],
                    message: 'Where to show this deployment block in the UI?',
                }])).category;
        }
        let integrateUrl;
        if ((_d = params === null || params === void 0 ? void 0 : params.info) === null || _d === void 0 ? void 0 : _d.integrateUrl) {
            integrateUrl = params.info.integrateUrl;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.integrateUrl) {
            integrateUrl = selectedBlock.integrateUrl;
        }
        else {
            integrateUrl = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'integrateUrl',
                    message: 'Do you have an integration URL (shown after deployment, e.g. your docs page), leave empty to skip',
                }])).integrateUrl;
            if (!integrateUrl) {
                integrateUrl = undefined;
            }
        }
        let cliArguments;
        if ((_e = params === null || params === void 0 ? void 0 : params.info) === null || _e === void 0 ? void 0 : _e.cliArguments) {
            cliArguments = params.info.cliArguments;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.integrateUrl) {
            cliArguments = selectedBlock.integrateUrl;
        }
        else {
            cliArguments = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'cliArguments',
                    message: 'CLI arguments to pass to this block (optional)',
                    default: '',
                }])).cliArguments;
        }
        let mountLearnBlock;
        if (typeof ((_f = params === null || params === void 0 ? void 0 : params.info) === null || _f === void 0 ? void 0 : _f.mountLearnBlock) === 'boolean') {
            mountLearnBlock = params.info.mountLearnBlock;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.mountLearnBlock) === 'boolean') {
            mountLearnBlock = selectedBlock.mountLearnBlock;
        }
        else {
            const resp = await inquirer_1.default.prompt({
                type: 'confirm',
                name: 'mountLearnBlock',
                message: `Mount learn block output under /data? ` +
                    `Useful if you need access to raw learn block output, only ` +
                    `supported if you have one learn block in your impulse.`,
                default: false,
            });
            mountLearnBlock = resp.mountLearnBlock;
        }
        let supportsEonCompiler;
        if (typeof ((_g = params === null || params === void 0 ? void 0 : params.info) === null || _g === void 0 ? void 0 : _g.supportsEonCompiler) === 'boolean') {
            supportsEonCompiler = params.info.supportsEonCompiler;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.supportsEonCompiler) === 'boolean') {
            supportsEonCompiler = selectedBlock.supportsEonCompiler;
        }
        else {
            const resp = await inquirer_1.default.prompt({
                type: 'confirm',
                name: 'supportsEonCompiler',
                message: `Does this deployment block support the EON Compiler? ` +
                    `If so, we'll pass EON Compiler output to this block.`,
                default: true,
            });
            supportsEonCompiler = resp.supportsEonCompiler;
        }
        let showOptimizations;
        if (typeof ((_h = params === null || params === void 0 ? void 0 : params.info) === null || _h === void 0 ? void 0 : _h.showOptimizations) === 'boolean') {
            showOptimizations = params.info.showOptimizations;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.showOptimizations) === 'boolean') {
            showOptimizations = selectedBlock.showOptimizations;
        }
        else {
            const resp = await inquirer_1.default.prompt({
                type: 'confirm',
                name: 'showOptimizations',
                message: `Show optimizations panel? ` +
                    `If disabled the optimizations panel (where you pick e.g. int8 quantized models) is hidden. ` +
                    `Disable this if your block operates on e.g. a SavedModel or ONNX file and does its own ` +
                    `optimization.`,
                default: true,
            });
            showOptimizations = resp.showOptimizations;
        }
        let privileged;
        if (typeof ((_j = params === null || params === void 0 ? void 0 : params.info) === null || _j === void 0 ? void 0 : _j.privileged) === 'boolean') {
            privileged = params.info.privileged;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.privileged) === 'boolean') {
            privileged = selectedBlock.privileged;
        }
        else {
            privileged = false;
        }
        if (!params) {
            params = {
                version: 1,
                type: 'deploy',
                info: {
                    name: blockName,
                    description: blockDescription,
                    category: category,
                    integrateUrl: integrateUrl,
                    mountLearnBlock: mountLearnBlock,
                    supportsEonCompiler: supportsEonCompiler,
                    showOptimizations: showOptimizations,
                    cliArguments: cliArguments,
                    privileged: privileged,
                }
            };
        }
        else {
            params.info.name = blockName;
            params.info.description = blockDescription;
            params.info.category = category;
            params.info.integrateUrl = integrateUrl;
            params.info.mountLearnBlock = mountLearnBlock;
            params.info.supportsEonCompiler = supportsEonCompiler;
            params.info.showOptimizations = showOptimizations;
            params.info.cliArguments = cliArguments;
            params.info.privileged = privileged;
        }
        await this._blockConfigManager.saveParameters(params);
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async initMachineLearningBlock(organizationInfo) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let existingBlocks = (await api.organizationBlocks.listOrganizationTransferLearningBlocks(organizationId)).transferLearningBlocks;
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        let params;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            params = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (Array.isArray(params)) {
                // old version, was only an array of parameters
                params = {
                    version: 1,
                    type: 'machine-learning',
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    info: {},
                    parameters: params,
                };
            }
            else {
                if (!params.version) {
                    params.version = 1;
                }
                params.type = 'machine-learning';
                params.info = params.info || {};
            }
        }
        let blockName;
        if ((_a = params === null || params === void 0 ? void 0 : params.info) === null || _a === void 0 ? void 0 : _a.name) {
            blockName = params.info.name;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.name) {
            blockName = selectedBlock.name;
        }
        else {
            blockName = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Enter the name of your block (min. 2 characters)',
                }])).name;
            if (blockName.length < 2) {
                throw new Error('New block must have a name longer than 2 characters.');
            }
        }
        let blockDescription;
        if ((_b = params === null || params === void 0 ? void 0 : params.info) === null || _b === void 0 ? void 0 : _b.description) {
            blockDescription = params.info.description;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.description) {
            blockDescription = selectedBlock.description;
        }
        else {
            blockDescription = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'description',
                    message: 'Enter the description of your block',
                }])).description;
        }
        let blockTlOperatesOn;
        if ((_c = params === null || params === void 0 ? void 0 : params.info) === null || _c === void 0 ? void 0 : _c.operatesOn) {
            blockTlOperatesOn = params.info.operatesOn;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.operatesOn) {
            blockTlOperatesOn = selectedBlock.operatesOn;
        }
        else {
            blockTlOperatesOn = (await inquirer_1.default.prompt([{
                    type: 'list',
                    name: 'operatesOn',
                    choices: [
                        {
                            name: 'Object Detection',
                            value: 'object_detection'
                        },
                        {
                            name: 'Image classification',
                            value: 'image'
                        },
                        {
                            name: 'Audio classification',
                            value: 'audio'
                        },
                        {
                            name: 'Classification',
                            value: 'other'
                        },
                        {
                            name: 'Regression',
                            value: 'regression'
                        },
                    ],
                    message: 'What type of data does this model operate on?',
                }])).operatesOn;
        }
        let blockTlImageInputScaling;
        if (blockTlOperatesOn === 'image' || blockTlOperatesOn === 'object_detection') {
            if ((_d = params === null || params === void 0 ? void 0 : params.info) === null || _d === void 0 ? void 0 : _d.imageInputScaling) {
                blockTlImageInputScaling = params.info.imageInputScaling;
            }
            else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.imageInputScaling) {
                blockTlImageInputScaling = selectedBlock.imageInputScaling;
            }
            else {
                blockTlImageInputScaling = (await inquirer_1.default.prompt([{
                        type: 'list',
                        name: 'inputScaling',
                        choices: organizationInfo.cliLists.imageInputScalingOptions.map(o => {
                            return {
                                name: o.label,
                                value: o.value,
                            };
                        }),
                        message: 'How is your input scaled?',
                        default: '0..1'
                    }])).inputScaling;
            }
        }
        let blockTlObjectDetectionLastLayer;
        if (blockTlOperatesOn === 'object_detection') {
            if ((_e = params === null || params === void 0 ? void 0 : params.info) === null || _e === void 0 ? void 0 : _e.objectDetectionLastLayer) {
                blockTlObjectDetectionLastLayer = params.info.objectDetectionLastLayer;
            }
            else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.objectDetectionLastLayer) {
                blockTlObjectDetectionLastLayer = selectedBlock.objectDetectionLastLayer;
            }
            else {
                blockTlObjectDetectionLastLayer = (await inquirer_1.default.prompt([{
                        type: 'list',
                        name: 'lastLayer',
                        choices: organizationInfo.cliLists.objectDetectionLastLayerOptions.map(o => {
                            return {
                                name: o.label,
                                value: o.value,
                            };
                        }),
                        message: `What's the last layer of this object detection model?`,
                    }])).lastLayer;
            }
        }
        let blockIndRequiresGpu;
        if (typeof ((_f = params === null || params === void 0 ? void 0 : params.info) === null || _f === void 0 ? void 0 : _f.indRequiresGpu) === 'boolean') {
            blockIndRequiresGpu = params.info.indRequiresGpu;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.indRequiresGpu) {
            blockIndRequiresGpu = selectedBlock.indRequiresGpu;
        }
        else {
            let blockTlCanRunWhere = (await inquirer_1.default.prompt([{
                    type: 'list',
                    name: 'canRunWhere',
                    default: 'cpu-or-gpu',
                    choices: [
                        {
                            name: 'Both CPU or GPU (default)',
                            value: 'cpu-or-gpu'
                        },
                        {
                            name: 'Only on GPU (GPUs are only available for enterprise projects)',
                            value: 'gpu'
                        },
                    ],
                    message: 'Where can your model train?',
                }])).canRunWhere;
            blockIndRequiresGpu = blockTlCanRunWhere === 'gpu';
        }
        let repoUrl;
        if ((_g = params === null || params === void 0 ? void 0 : params.info) === null || _g === void 0 ? void 0 : _g.repositoryUrl) {
            repoUrl = params.info.repositoryUrl;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.repositoryUrl) {
            repoUrl = selectedBlock.repositoryUrl;
        }
        else {
            repoUrl = await (0, blocks_helper_1.guessRepoUrl)();
        }
        let customModelVariants;
        if ((_h = params === null || params === void 0 ? void 0 : params.info) === null || _h === void 0 ? void 0 : _h.customModelVariants) {
            customModelVariants = params.info.customModelVariants;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.customModelVariants) {
            customModelVariants = selectedBlock.customModelVariants;
        }
        let displayCategory;
        if ((_j = params === null || params === void 0 ? void 0 : params.info) === null || _j === void 0 ? void 0 : _j.displayCategory) {
            displayCategory = params.info.displayCategory;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.displayCategory) {
            displayCategory = selectedBlock.displayCategory;
        }
        if (!params) {
            params = {
                version: 1,
                type: 'machine-learning',
                info: {
                    name: blockName,
                    description: blockDescription,
                    operatesOn: blockTlOperatesOn,
                    imageInputScaling: blockTlImageInputScaling,
                    indRequiresGpu: blockIndRequiresGpu,
                    objectDetectionLastLayer: blockTlObjectDetectionLastLayer,
                    repositoryUrl: repoUrl,
                    customModelVariants: customModelVariants,
                    displayCategory: displayCategory,
                },
                parameters: [],
            };
        }
        else {
            params.info.name = blockName;
            params.info.description = blockDescription;
            params.info.operatesOn = blockTlOperatesOn;
            params.info.imageInputScaling = blockTlImageInputScaling;
            params.info.indRequiresGpu = blockIndRequiresGpu;
            params.info.objectDetectionLastLayer = blockTlObjectDetectionLastLayer;
            params.info.repositoryUrl = repoUrl;
            params.info.customModelVariants = customModelVariants;
            params.info.displayCategory = displayCategory;
        }
        await this._blockConfigManager.saveParameters(params);
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async initSyntheticDataBlock(organizationInfo) {
        var _a, _b;
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let existingBlocks = (await api.organizationBlocks.listOrganizationTransformationBlocks(organizationId)).transformationBlocks
            .filter(x => x.showInSyntheticData);
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        let params;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            params = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (Array.isArray(params)) {
                // old version, was only an array of parameters
                params = {
                    version: 1,
                    type: 'synthetic-data',
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    info: {},
                    parameters: params,
                };
            }
            else {
                if (!params.version) {
                    params.version = 1;
                }
                params.type = 'synthetic-data';
                params.info = params.info || {};
            }
        }
        let blockName;
        if ((_a = params === null || params === void 0 ? void 0 : params.info) === null || _a === void 0 ? void 0 : _a.name) {
            blockName = params.info.name;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.name) {
            blockName = selectedBlock.name;
        }
        else {
            blockName = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Enter the name of your block (min. 2 characters)',
                }])).name;
            if (blockName.length < 2) {
                throw new Error('New block must have a name longer than 2 characters.');
            }
        }
        let blockDescription;
        if ((_b = params === null || params === void 0 ? void 0 : params.info) === null || _b === void 0 ? void 0 : _b.description) {
            blockDescription = params.info.description;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.description) {
            blockDescription = selectedBlock.description;
        }
        else {
            blockDescription = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'description',
                    message: 'Enter the description of your block',
                }])).description;
        }
        if (!params) {
            params = {
                version: 1,
                type: 'synthetic-data',
                info: {
                    name: blockName,
                    description: blockDescription,
                    requiredEnvVariables: undefined,
                },
                parameters: [],
            };
        }
        else {
            params.info.name = blockName;
            params.info.description = blockDescription;
        }
        await this._blockConfigManager.saveParameters(params);
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async initAIActionBlock(organizationInfo) {
        var _a, _b, _c;
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let existingBlocks = (await api.organizationBlocks.listOrganizationTransformationBlocks(organizationId)).transformationBlocks
            .filter(x => x.showInAIActions);
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        let params;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            params = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (Array.isArray(params)) {
                // old version, was only an array of parameters
                params = {
                    version: 1,
                    type: 'ai-action',
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    info: {},
                    parameters: params,
                };
            }
            else {
                if (!params.version) {
                    params.version = 1;
                }
                params.type = 'ai-action';
                params.info = params.info || {};
            }
        }
        let blockName;
        if ((_a = params === null || params === void 0 ? void 0 : params.info) === null || _a === void 0 ? void 0 : _a.name) {
            blockName = params.info.name;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.name) {
            blockName = selectedBlock.name;
        }
        else {
            blockName = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Enter the name of your block (min. 2 characters)',
                }])).name;
            if (blockName.length < 2) {
                throw new Error('New block must have a name longer than 2 characters.');
            }
        }
        let blockDescription;
        if ((_b = params === null || params === void 0 ? void 0 : params.info) === null || _b === void 0 ? void 0 : _b.description) {
            blockDescription = params.info.description;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.description) {
            blockDescription = selectedBlock.description;
        }
        else {
            blockDescription = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'description',
                    message: 'Enter the description of your block',
                }])).description;
        }
        let operatesOn;
        if ((_c = params === null || params === void 0 ? void 0 : params.info) === null || _c === void 0 ? void 0 : _c.operatesOn) {
            operatesOn = params.info.operatesOn;
        }
        else {
            let choicesMap = {
                images_object_detection: 'Images (object detection)',
                images_single_label: 'Images (single label per image)',
                audio: 'Audio',
                other: 'Other data',
            };
            operatesOn = (await inquirer_1.default.prompt([{
                    type: 'checkbox',
                    name: 'operatesOn',
                    choices: Object.keys(choicesMap).map(k => {
                        return {
                            name: choicesMap[k],
                            value: k,
                        };
                    }),
                    message: 'What data does this block operate on (can select multiple)?',
                }])).operatesOn;
        }
        if (operatesOn.length === 0) {
            operatesOn = [];
        }
        if (!params) {
            params = {
                version: 1,
                type: 'ai-action',
                info: {
                    name: blockName,
                    description: blockDescription,
                    requiredEnvVariables: undefined,
                    operatesOn: operatesOn,
                },
                parameters: [],
            };
        }
        else {
            params.info.name = blockName;
            params.info.description = blockDescription;
            params.info.operatesOn = operatesOn;
        }
        await this._blockConfigManager.saveParameters(params);
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async initTransformBlock(organizationInfo) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const organizationId = organizationInfo.organization.id;
        const api = this._config.api;
        let existingBlocks = (await api.organizationBlocks.listOrganizationTransformationBlocks(organizationId)).transformationBlocks
            .filter(x => !x.showInAIActions && !x.showInSyntheticData);
        let createOrUpdateInqRes = await this.createOrUpdate(existingBlocks);
        let selectedBlock;
        if (createOrUpdateInqRes === 'update') {
            selectedBlock = await this.selectExistingBlock(existingBlocks);
        }
        let params;
        if (await (0, blocks_helper_1.pathExists)(this._paths.parametersJson)) {
            params = JSON.parse(await fs_1.default.promises.readFile(this._paths.parametersJson, 'utf-8'));
            if (Array.isArray(params)) {
                // old version, was only an array of parameters
                params = {
                    version: 1,
                    type: 'transform',
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    info: {},
                    parameters: params,
                };
            }
            else {
                if (!params.version) {
                    params.version = 1;
                }
                params.type = 'transform';
                params.info = params.info || {};
            }
        }
        let blockName;
        if ((_a = params === null || params === void 0 ? void 0 : params.info) === null || _a === void 0 ? void 0 : _a.name) {
            blockName = params.info.name;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.name) {
            blockName = selectedBlock.name;
        }
        else {
            blockName = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Enter the name of your block (min. 2 characters)',
                }])).name;
            if (blockName.length < 2) {
                throw new Error('New block must have a name longer than 2 characters.');
            }
        }
        let blockDescription;
        if ((_b = params === null || params === void 0 ? void 0 : params.info) === null || _b === void 0 ? void 0 : _b.description) {
            blockDescription = params.info.description;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.description) {
            blockDescription = selectedBlock.description;
        }
        else {
            blockDescription = (await inquirer_1.default.prompt([{
                    type: 'input',
                    name: 'description',
                    message: 'Enter the description of your block',
                }])).description;
        }
        let blockOperatesOn;
        if ((_c = params === null || params === void 0 ? void 0 : params.info) === null || _c === void 0 ? void 0 : _c.operatesOn) {
            blockOperatesOn = params.info.operatesOn;
        }
        else if (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.operatesOn) {
            blockOperatesOn = selectedBlock.operatesOn;
        }
        else {
            blockOperatesOn = (await inquirer_1.default.prompt([{
                    type: 'list',
                    name: 'operatesOn',
                    choices: [
                        {
                            name: 'File (--in-file passed into the block)',
                            value: 'file'
                        },
                        {
                            name: 'Directory (--in-directory passed into the block)',
                            value: 'directory'
                        },
                        {
                            name: 'Standalone (runs the container, but no files / directories passed in)',
                            value: 'standalone'
                        }
                    ],
                    message: 'What type of data does this block operate on?',
                }])).operatesOn;
        }
        let transformMountpoints;
        if ((_d = params === null || params === void 0 ? void 0 : params.info) === null || _d === void 0 ? void 0 : _d.transformMountpoints) {
            transformMountpoints = params.info.transformMountpoints;
        }
        else if (selectedBlock) {
            transformMountpoints = selectedBlock.additionalMountPoints.filter(mp => mp.bucketId).map(mp => {
                return {
                    bucketId: mp.bucketId,
                    mountPoint: mp.mountPoint,
                };
            });
        }
        else {
            let buckets = await this._config.api.organizationData.listOrganizationBuckets(organizationId);
            if (buckets.buckets && buckets.buckets.length > 0) {
                transformMountpoints = (await inquirer_1.default.prompt([{
                        type: 'checkbox',
                        name: 'buckets',
                        choices: buckets.buckets.map(x => {
                            return {
                                name: x.bucket,
                                value: x.id.toString()
                            };
                        }),
                        message: 'Which buckets do you want to mount into this block ' +
                            '(will be mounted under /mnt/s3fs/BUCKET_NAME, you can change these mount points in the Studio)?',
                    }])).buckets.map(y => {
                    var _a;
                    let b = (_a = buckets.buckets) === null || _a === void 0 ? void 0 : _a.find(z => z.id === Number(y));
                    return {
                        bucketId: Number(y),
                        mountPoint: b ? ('/mnt/s3fs/' + (b === null || b === void 0 ? void 0 : b.bucket)) : '',
                    };
                }).filter(x => !!x.mountPoint && !isNaN(x.bucketId));
            }
            else {
                transformMountpoints = [];
            }
        }
        let indMetadata;
        if (typeof ((_e = params === null || params === void 0 ? void 0 : params.info) === null || _e === void 0 ? void 0 : _e.indMetadata) === 'boolean') {
            indMetadata = params.info.indMetadata;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.indMetadata) === 'boolean') {
            indMetadata = selectedBlock.indMetadata;
        }
        let cliArguments;
        if (typeof ((_f = params === null || params === void 0 ? void 0 : params.info) === null || _f === void 0 ? void 0 : _f.cliArguments) === 'string') {
            cliArguments = params.info.cliArguments;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.cliArguments) === 'string') {
            cliArguments = selectedBlock.cliArguments;
        }
        let allowExtraCliArguments;
        if (typeof ((_g = params === null || params === void 0 ? void 0 : params.info) === null || _g === void 0 ? void 0 : _g.allowExtraCliArguments) === 'boolean') {
            allowExtraCliArguments = params.info.allowExtraCliArguments;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.allowExtraCliArguments) === 'boolean') {
            allowExtraCliArguments = selectedBlock.allowExtraCliArguments;
        }
        let showInDataSources;
        if (typeof ((_h = params === null || params === void 0 ? void 0 : params.info) === null || _h === void 0 ? void 0 : _h.showInDataSources) === 'boolean') {
            showInDataSources = params.info.showInDataSources;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.showInDataSources) === 'boolean') {
            showInDataSources = selectedBlock.showInDataSources;
        }
        let showInCreateTransformationJob;
        if (typeof ((_j = params === null || params === void 0 ? void 0 : params.info) === null || _j === void 0 ? void 0 : _j.showInCreateTransformationJob) === 'boolean') {
            showInCreateTransformationJob = params.info.showInCreateTransformationJob;
        }
        else if (typeof (selectedBlock === null || selectedBlock === void 0 ? void 0 : selectedBlock.showInCreateTransformationJob) === 'boolean') {
            showInCreateTransformationJob = selectedBlock.showInCreateTransformationJob;
        }
        if (!params) {
            params = {
                version: 1,
                type: 'transform',
                info: {
                    name: blockName,
                    description: blockDescription,
                    operatesOn: blockOperatesOn,
                    transformMountpoints: transformMountpoints,
                    allowExtraCliArguments: allowExtraCliArguments,
                    cliArguments: cliArguments,
                    indMetadata: indMetadata,
                    showInCreateTransformationJob: showInCreateTransformationJob,
                    showInDataSources: showInDataSources,
                    requiredEnvVariables: undefined,
                },
                parameters: [],
            };
        }
        else {
            params.info.name = blockName;
            params.info.description = blockDescription;
            params.info.operatesOn = blockOperatesOn;
            params.info.transformMountpoints = transformMountpoints;
            params.info.allowExtraCliArguments = allowExtraCliArguments;
            params.info.cliArguments = cliArguments;
            params.info.indMetadata = indMetadata;
            params.info.showInCreateTransformationJob = showInCreateTransformationJob;
            params.info.showInDataSources = showInDataSources;
        }
        await this._blockConfigManager.saveParameters(params);
        await this._blockConfigManager.saveConfig({
            organizationId: organizationId,
            id: selectedBlock ? selectedBlock.id : undefined,
        });
    }
    async createOrUpdate(existingBlocks) {
        return existingBlocks.length > 0 ? (await inquirer_1.default.prompt([{
                type: 'list',
                choices: [
                    {
                        name: 'Create a new block',
                        value: 'create'
                    },
                    {
                        name: 'Update an existing block',
                        value: 'update'
                    }
                ],
                name: 'option',
                message: 'Choose an option',
                pageSize: 20
            }])).option : 'create';
    }
    async selectExistingBlock(existingBlocks) {
        // Update an existing block
        // Choose a block ID
        let blockChoiceInqRes = await inquirer_1.default.prompt([{
                type: 'list',
                choices: existingBlocks,
                name: 'id',
                message: 'Choose a block to update',
                pageSize: 20
            }]);
        const ret = existingBlocks.find(block => block.name === blockChoiceInqRes.id);
        if (!ret) {
            throw new Error('selectExistingBlock failed to find "' + blockChoiceInqRes.id + '"');
        }
        return ret;
    }
}
exports.InitCLIBlock = InitCLIBlock;
//# sourceMappingURL=init-cli-block.js.map