"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GStreamer = void 0;
const child_process_1 = require("child_process");
const tsee_1 = require("tsee");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const spawn_helper_1 = require("./spawn-helper");
const util_1 = __importDefault(require("util"));
const crypto_1 = __importDefault(require("crypto"));
const argv_split_1 = require("../argv-split");
const PREFIX = '\x1b[34m[GST]\x1b[0m';
const CUSTOM_GST_LAUNCH_COMMAND = 'custom-gst-launch-command';
const DEFAULT_GST_VIDEO_SOURCE = 'v4l2src';
class GStreamer extends tsee_1.EventEmitter {
    constructor(verbose, options) {
        super();
        this._handledFiles = {};
        this._lastHash = '';
        this._processing = false;
        this._mode = 'default';
        this._isStarted = false;
        this._isRestarting = false;
        this._verbose = verbose;
        this._customLaunchCommand = options === null || options === void 0 ? void 0 : options.customLaunchCommand;
        this._spawnHelper = (options === null || options === void 0 ? void 0 : options.spawnHelperOverride) || spawn_helper_1.spawnHelper;
        this._modeOverride = options === null || options === void 0 ? void 0 : options.modeOverride;
    }
    async init() {
        try {
            await this._spawnHelper('which', ['gst-launch-1.0']);
        }
        catch (ex) {
            throw new Error('Missing "gst-launch-1.0" in PATH. Install via `sudo apt install -y gstreamer1.0-tools gstreamer1.0-plugins-good gstreamer1.0-plugins-base gstreamer1.0-plugins-base-apps`');
        }
        try {
            await this._spawnHelper('which', ['gst-device-monitor-1.0']);
        }
        catch (ex) {
            throw new Error('Missing "gst-device-monitor-1.0" in PATH. Install via `sudo apt install -y gstreamer1.0-tools gstreamer1.0-plugins-good gstreamer1.0-plugins-base gstreamer1.0-plugins-base-apps`');
        }
        let osRelease;
        if (await this.exists('/etc/os-release')) {
            console.log(PREFIX, 'checking for /etc/os-release');
            osRelease = await fs_1.default.promises.readFile('/etc/os-release', 'utf-8');
        }
        let firmwareModel;
        // using /proc/device-tree as recommended in user space.
        if (await this.exists('/proc/device-tree/model')) {
            firmwareModel = await fs_1.default.promises.readFile('/proc/device-tree/model', 'utf-8');
        }
        if (osRelease &&
            ((osRelease.indexOf('bullseye') > -1)
                || (osRelease.indexOf('bookworm') > -1))) {
            if (osRelease.indexOf('ID=raspbian') > -1) {
                this._mode = 'rpi';
            }
            if (firmwareModel && firmwareModel.indexOf('Raspberry Pi') > -1) {
                this._mode = 'rpi';
            }
        }
        if (firmwareModel && firmwareModel.indexOf('Microchip SAMA7G5') > -1) {
            this._mode = 'microchip';
        }
        this._mode = (this._modeOverride) ? this._modeOverride : this._mode;
    }
    async listDevices() {
        if (this._customLaunchCommand) {
            return [CUSTOM_GST_LAUNCH_COMMAND];
        }
        let devices = await this.getAllDevices();
        if (this._verbose) {
            console.log(PREFIX, 'Found devices:', JSON.stringify(devices, null, 2));
        }
        return devices.map(d => d.name);
    }
    async start(options) {
        if (this._captureProcess) {
            throw new Error('Capture was already started');
        }
        this._lastOptions = options;
        this._handledFiles = {};
        let dimensions = options.dimensions || { width: 640, height: 480 };
        // if we have /dev/shm, use that (RAM backed, instead of SD card backed, better for wear)
        let osTmpDir = os_1.default.tmpdir();
        if (await this.exists('/dev/shm')) {
            osTmpDir = '/dev/shm';
        }
        this._tempDir = await fs_1.default.promises.mkdtemp(path_1.default.join(osTmpDir, 'edge-impulse-cli'));
        const device = (await this.getAllDevices()).find(d => d.name === options.device);
        if (!device) {
            throw new Error('Invalid device ' + options.device);
        }
        if (device.caps.length === 0) {
            throw new Error('Could not find resolution info for this device');
        }
        if (device.name === 'RZG2L_CRU') {
            // run commands to initialize the Coral camera on Renesas
            if (this._verbose) {
                console.log(PREFIX, 'Detected RZ/G2L target, initializing camera...');
            }
            await (0, spawn_helper_1.spawnHelper)('media-ctl', ['-d', '/dev/media0', '-r']);
            await (0, spawn_helper_1.spawnHelper)('media-ctl', ['-d', '/dev/media0', '-l', "'rzg2l_csi2 10830400.csi2':1 -> 'CRU output':0 [1]"]);
            await (0, spawn_helper_1.spawnHelper)('media-ctl', ['-d', '/dev/media0', '-V', "'rzg2l_csi2 10830400.csi2':1 [fmt:UYVY8_2X8/640x480 field:none]"]);
            await (0, spawn_helper_1.spawnHelper)('media-ctl', ['-d', '/dev/media0', '-V', "'ov5645 0-003c':0 [fmt:UYVY8_2X8/640x480 field:none]"]);
            if (this._verbose) {
                console.log(PREFIX, 'Detected RZ/G2L target, initializing camera OK');
            }
        }
        const { invokeProcess, command, args } = await this.getGstreamerLaunchCommand(device, dimensions);
        if (this._verbose) {
            console.log(PREFIX, `Starting ${command} with`, args);
        }
        if (invokeProcess === 'spawn') {
            this._captureProcess = (0, child_process_1.spawn)(command, args, { env: process.env, cwd: this._tempDir });
        }
        else if (invokeProcess === 'exec') {
            this._captureProcess = (0, child_process_1.exec)(command + ' ' + args.join(' '), { env: process.env, cwd: this._tempDir });
        }
        else {
            throw new Error('Invalid value for invokeProcess');
        }
        if (this._captureProcess && this._captureProcess.stdout && this._captureProcess.stderr &&
            this._verbose) {
            this._captureProcess.stdout.on('data', (d) => {
                console.log(PREFIX, d.toString('utf-8'));
            });
            this._captureProcess.stderr.on('data', (d) => {
                console.log(PREFIX, d.toString('utf-8'));
            });
        }
        let lastPhoto = 0;
        let nextFrame = Date.now();
        this._watcher = fs_1.default.watch(this._tempDir, async (eventType, fileName) => {
            if (eventType !== 'rename')
                return;
            if (fileName === null)
                return;
            if (!(fileName.endsWith('.jpeg') || fileName.endsWith('.jpg')))
                return;
            if (!this._tempDir)
                return;
            if (this._handledFiles[fileName])
                return;
            // not next frame yet?
            if (this._processing || Date.now() < nextFrame) {
                this._handledFiles[fileName] = true;
                if (await this.exists(path_1.default.join(this._tempDir, fileName))) {
                    await fs_1.default.promises.unlink(path_1.default.join(this._tempDir, fileName));
                }
                return;
            }
            nextFrame = Date.now() + options.intervalMs;
            try {
                this._processing = true;
                this._handledFiles[fileName] = true;
                if (lastPhoto !== 0 && this._verbose) {
                    console.log(PREFIX, 'Got snapshot', fileName, 'time since last:', (Date.now() - lastPhoto) + 'ms.', 'size');
                }
                if (this._keepAliveTimeout) {
                    clearTimeout(this._keepAliveTimeout);
                }
                try {
                    let data = await fs_1.default.promises.readFile(path_1.default.join(this._tempDir, fileName));
                    // hash not changed? don't emit another event (streamer does this on Rpi)
                    let hash = crypto_1.default.createHash('sha256').update(data).digest('hex');
                    if (hash !== this._lastHash) {
                        this.emit('snapshot', data, path_1.default.basename(fileName));
                        lastPhoto = Date.now();
                        // 2 seconds no new data? trigger timeout
                        if (this._keepAliveTimeout) {
                            clearTimeout(this._keepAliveTimeout);
                        }
                        this._keepAliveTimeout = setTimeout(() => {
                            // eslint-disable-next-line @typescript-eslint/no-floating-promises
                            this.timeoutCallback();
                        }, 2000);
                    }
                    else if (this._verbose) {
                        console.log(PREFIX, 'Discarding', fileName, 'hash does not differ');
                    }
                    this._lastHash = hash;
                }
                catch (ex) {
                    console.error('Failed to load file', path_1.default.join(this._tempDir, fileName), ex);
                }
                if (await this.exists(path_1.default.join(this._tempDir, fileName))) {
                    await fs_1.default.promises.unlink(path_1.default.join(this._tempDir, fileName));
                }
            }
            finally {
                this._processing = false;
            }
        });
        let p = new Promise((resolve, reject) => {
            if (this._captureProcess) {
                let cp = this._captureProcess;
                this._captureProcess.on('close', code => {
                    if (this._keepAliveTimeout) {
                        clearTimeout(this._keepAliveTimeout);
                    }
                    if (typeof code === 'number') {
                        reject('Capture process failed with code ' + code);
                    }
                    else {
                        reject('Failed to start capture process, but no exit code. ' +
                            'This might be a permissions issue. ' +
                            'Are you running this command from a simulated shell (like in Visual Studio Code)?');
                    }
                    // already started and we're the active process?
                    if (this._isStarted && cp === this._captureProcess && !this._isRestarting) {
                        this.emit('error', 'gstreamer process was killed with code (' + code + ')');
                    }
                    this._captureProcess = undefined;
                });
            }
            // eslint-disable-next-line @typescript-eslint/no-floating-promises
            (async () => {
                if (!this._tempDir) {
                    throw new Error('tempDir is undefined');
                }
                const watcher = fs_1.default.watch(this._tempDir, () => {
                    this._isStarted = true;
                    resolve();
                    watcher.close();
                });
                setTimeout(async () => {
                    if (this._keepAliveTimeout) {
                        clearTimeout(this._keepAliveTimeout);
                    }
                    return reject('First photo was not created within 20 seconds');
                }, 20000);
            })();
        });
        p.catch(() => this.stop());
        return p;
    }
    async getGstreamerLaunchCommand(device, dimensions) {
        if (device.id === CUSTOM_GST_LAUNCH_COMMAND) {
            if (!this._customLaunchCommand) {
                throw new Error('_customLaunchCommand is null');
            }
            let customArgs = (0, argv_split_1.split)(this._customLaunchCommand);
            customArgs = customArgs.concat([
                `!`,
                `multifilesink`,
                `location=test%05d.jpg`
            ]);
            return {
                invokeProcess: 'spawn',
                command: 'gst-launch-1.0',
                args: customArgs,
            };
        }
        // now we need to determine the resolution... we want something as close as possible to dimensions.widthx480
        let caps = device.caps.filter(c => {
            return c.width >= dimensions.width && c.height >= dimensions.height;
        }).sort((a, b) => {
            let diffA = Math.abs(a.width - dimensions.width) + Math.abs(a.height - dimensions.height);
            let diffB = Math.abs(b.width - dimensions.width) + Math.abs(b.height - dimensions.height);
            return diffA - diffB;
        });
        // if the device supports video/x-raw, only list those types
        const videoCaps = caps.filter(x => x.type === 'video/x-raw');
        if (videoCaps.length > 0) {
            caps = videoCaps;
        }
        // choose the top of the list
        let cap = caps[0];
        if (!cap) {
            cap = {
                type: 'video/x-raw',
                width: dimensions.width,
                height: dimensions.height,
                framerate: 30,
            };
        }
        let videoSource = [device.videoSource, 'device=' + device.id];
        if ((this._mode === 'rpi') || (this._mode === 'microchip')) {
            // Rpi camera
            if ((!device.id)
                || (device.name.indexOf('unicam') > -1)
                || (device.name.indexOf('bcm2835-isp') > -1)) {
                videoSource = ['libcamerasrc'];
                const hasPlugin = await this.hasGstPlugin('libcamerasrc');
                if (!hasPlugin) {
                    throw new Error('Missing "libcamerasrc" gstreamer element. Install via `sudo apt install -y gstreamer1.0-libcamera`');
                }
            }
        }
        if (device.id === 'pylonsrc') {
            videoSource = ['pylonsrc'];
        }
        let invokeProcess;
        let args;
        if ((cap.type === 'video/x-raw') || (cap.type === 'pylonsrc')) {
            if (this._mode === 'rpi' && device.name.indexOf('arducam') > -1) {
                videoSource = videoSource.concat([
                    `!`,
                    `video/x-raw,width=${cap.width},height=${cap.height},format=YUY2`
                ]);
            }
            else {
                videoSource = videoSource.concat([
                    `!`,
                    `video/x-raw,width=${cap.width},height=${cap.height}`
                ]);
            }
            args = videoSource.concat([
                `!`,
                `videoconvert`,
                `!`,
                `jpegenc`,
                `!`,
                `multifilesink`,
                `location=test%05d.jpg`
            ]);
            invokeProcess = 'spawn';
        }
        else if (cap.type === 'image/jpeg') {
            args = videoSource.concat([
                `!`,
                `image/jpeg,width=${cap.width},height=${cap.height}`,
                `!`,
                `multifilesink`,
                `location=test%05d.jpg`
            ]);
            invokeProcess = 'spawn';
        }
        else if (cap.type === 'nvarguscamerasrc') {
            args = [
                `nvarguscamerasrc ! "video/x-raw(memory:NVMM),width=${cap.width},height=${cap.height}" ! ` +
                    `nvvidconv flip-method=0 ! video/x-raw,width=${cap.width},height=${cap.height} ! nvvidconv ! ` +
                    `jpegenc ! multifilesink location=test%05d.jpg`
            ];
            // no idea why... but if we throw this thru `spawn` this yields an invalid pipeline...
            invokeProcess = 'exec';
        }
        else {
            throw new Error('Invalid cap type ' + cap.type);
        }
        return {
            invokeProcess,
            command: 'gst-launch-1.0',
            args,
        };
    }
    async stop() {
        if (this._keepAliveTimeout) {
            clearTimeout(this._keepAliveTimeout);
        }
        let stopRes = new Promise((resolve) => {
            if (this._captureProcess) {
                this._captureProcess.on('close', code => {
                    if (this._watcher) {
                        this._watcher.on('close', async () => {
                            if (this._tempDir) {
                                const files = await fs_1.default.promises.readdir(this._tempDir);
                                const imageFiles = files.filter(file => {
                                    const fileExt = path_1.default.extname(file).toLowerCase();
                                    return (fileExt === 'jpg' || fileExt === 'jpeg');
                                });
                                for (const file of imageFiles) {
                                    await fs_1.default.promises.unlink(path_1.default.join(this._tempDir, file));
                                }
                            }
                        });
                        this._watcher.close();
                    }
                    resolve();
                });
                this._captureProcess.kill('SIGINT');
                setTimeout(() => {
                    if (this._captureProcess) {
                        this._captureProcess.kill('SIGHUP');
                    }
                }, 3000);
            }
            else {
                resolve();
            }
        });
        // set isStarted to false
        stopRes
            .then(() => { this._isStarted = false; })
            .catch(() => { this._isStarted = false; });
        return stopRes;
    }
    async getAllDevices() {
        if (this._customLaunchCommand) {
            let width = 640;
            let height = 480;
            let widthM = CUSTOM_GST_LAUNCH_COMMAND.match(/width=(\d+)/);
            if (widthM && widthM.length >= 1) {
                width = Number(widthM[1]);
            }
            let heightM = CUSTOM_GST_LAUNCH_COMMAND.match(/height=(\d+)/);
            if (heightM && heightM.length >= 1) {
                height = Number(heightM[1]);
            }
            return [{
                    id: CUSTOM_GST_LAUNCH_COMMAND,
                    name: CUSTOM_GST_LAUNCH_COMMAND,
                    caps: [{
                            type: 'video/x-raw',
                            framerate: 60,
                            width: width,
                            height: height,
                        }],
                    videoSource: DEFAULT_GST_VIDEO_SOURCE,
                }];
        }
        let lines;
        try {
            lines = (await this._spawnHelper('gst-device-monitor-1.0', []))
                .split('\n').filter(x => !!x).map(x => x.trim());
        }
        catch (ex2) {
            const ex = ex2;
            const message = ex.message || ex.toString();
            if (typeof message === 'string' && ((message.indexOf('Failed to start device monitor') > -1) ||
                (message.indexOf('Failed to query video capabilities') > -1))) {
                if (this._verbose) {
                    console.log(PREFIX, 'Failed to start gst-device-monitor-1.0, retrying with only video sources...');
                }
                lines = (await this._spawnHelper('gst-device-monitor-1.0', ['Video/Source', 'Source/Video', 'Video/CameraSource']))
                    .split('\n').filter(x => !!x).map(x => x.trim());
            }
            else {
                throw ex;
            }
        }
        let devices = [];
        let currDevice;
        for (let l of lines) {
            if (l === 'Device found:') {
                if (currDevice) {
                    devices.push(currDevice);
                }
                currDevice = {
                    name: '',
                    deviceClass: '',
                    rawCaps: [],
                    inCapMode: false,
                    id: '',
                    videoSource: DEFAULT_GST_VIDEO_SOURCE,
                    caps: []
                };
                continue;
            }
            if (!currDevice)
                continue;
            if (l.startsWith('name  :')) {
                currDevice.name = l.split(':')[1].trim();
                continue;
            }
            if (l.startsWith('class :')) {
                currDevice.deviceClass = l.split(':')[1].trim();
                continue;
            }
            if (l.startsWith('caps  :')) {
                let cap = l.split(':')[1].trim();
                currDevice.rawCaps.push(cap);
                currDevice.inCapMode = true;
                continue;
            }
            if (l.startsWith('properties:')) {
                currDevice.inCapMode = false;
                continue;
            }
            if (currDevice.inCapMode) {
                currDevice.rawCaps.push(l);
            }
            if (l.startsWith('device.path =')) {
                currDevice.id = l.split('=')[1].trim();
            }
            if (l.startsWith('api.v4l2.path =') && currDevice.id === '') {
                currDevice.id = l.split('=')[1].trim();
            }
            if (l.startsWith('gst-launch-1.0 ')) {
                const videoSource = l.split(' ')[1].trim();
                if (videoSource !== '...') {
                    // Can be in the form a.b,
                    // e.g uvch264src.vfsrc
                    currDevice.videoSource = videoSource.split('.')[0].trim();
                    if (currDevice.videoSource === 'pipewiresrc') {
                        currDevice.videoSource = DEFAULT_GST_VIDEO_SOURCE;
                    }
                }
            }
        }
        if (currDevice) {
            devices.push(currDevice);
        }
        for (let d of devices) {
            let c = d.rawCaps.filter(x => x.startsWith('video/x-raw') || x.startsWith('image/jpeg')).map(l => {
                let width = (l.match(/width=[^\d]+(\d+)/) || [])[1];
                let height = (l.match(/height=[^\d]+(\d+)/) || [])[1];
                let framerate = (l.match(/framerate=[^\d]+(\d+)/) || [])[1];
                // Rpi on bullseye has lines like this..
                // eslint-disable-next-line @stylistic/max-len
                // image/jpeg, width=160, height=120, pixel-aspect-ratio=1/1, framerate={ (fraction)30/1, (fraction)24/1, (fraction)20/1, (fraction)15/1, (fraction)10/1, (fraction)15/2, (fraction)5/1 }
                if (!width) {
                    width = (l.match(/width=(\d+)/) || [])[1];
                }
                if (!height) {
                    height = (l.match(/height=(\d+)/) || [])[1];
                }
                if (!framerate) {
                    framerate = (l.match(/framerate=(\d+)/) || [])[1];
                }
                let r = {
                    type: l.startsWith('video/x-raw') ? 'video/x-raw' : 'image/jpeg',
                    width: Number(width || '0'),
                    height: Number(height || '0'),
                    framerate: Number(framerate || '0'),
                };
                return r;
            });
            if (this._mode === 'rpi' || this._mode === 'microchip') { // no framerate here...
                c = c.filter(x => x.width && x.height);
            }
            else if (d.name === 'RZG2L_CRU') {
                // so here we always parse 320x240, but 640x480 is also fine
                c = c.filter(x => x.width && x.height).map(x => {
                    if (x.width === 320) {
                        x.width = 640;
                    }
                    if (x.height === 240) {
                        x.height = 480;
                    }
                    return x;
                });
            }
            else {
                c = c.filter(x => x.width && x.height && x.framerate);
            }
            c = c.reduce((curr, o) => {
                // deduplicate caps
                if (!curr.some(obj => obj.framerate === o.framerate &&
                    obj.width === o.width &&
                    obj.height === o.height &&
                    obj.framerate === o.framerate)) {
                    curr.push(o);
                }
                return curr;
            }, []);
            d.caps = c;
        }
        devices = devices.filter(d => {
            return (d.deviceClass === 'Video/Source' ||
                d.deviceClass === 'Source/Video' ||
                d.deviceClass === 'Video/CameraSource') &&
                d.caps.length > 0;
        });
        // NVIDIA has their own plugins, query them too
        devices = devices.concat(await this.listNvarguscamerasrcDevices());
        devices = devices.concat(await this.listPylonsrcDevices());
        let mapped = devices.map(d => {
            let name = devices.filter(x => x.name === d.name).length >= 2 ?
                d.name + ' (' + d.id + ')' :
                d.name;
            return {
                id: d.id,
                name: name,
                caps: d.caps,
                videoSource: d.videoSource,
            };
        });
        // deduplicate (by id)
        mapped = mapped.reduce((curr, m) => {
            if (curr.find(x => x.id === m.id))
                return curr;
            curr.push(m);
            return curr;
        }, []);
        return mapped;
    }
    async hasGstPlugin(plugin) {
        let hasPlugin;
        try {
            hasPlugin = (await this._spawnHelper('gst-inspect-1.0', [])).indexOf(plugin) > -1;
        }
        catch (ex) {
            if (this._verbose) {
                console.log(PREFIX, 'Error invoking gst-inspect-1.0:', ex);
            }
            hasPlugin = false;
        }
        return hasPlugin;
    }
    async listNvarguscamerasrcDevices() {
        const hasPlugin = await this.hasGstPlugin('nvarguscamerasrc');
        if (!hasPlugin) {
            return [];
        }
        let caps = [];
        let gstLaunchRet;
        // not overridden spawn helper?
        if (this._spawnHelper === spawn_helper_1.spawnHelper) {
            gstLaunchRet = await new Promise((resolve, reject) => {
                let command = 'gst-launch-1.0';
                let args = ['nvarguscamerasrc'];
                let opts = { ignoreErrors: true };
                const p = (0, child_process_1.spawn)(command, args, { env: process.env });
                let allData = [];
                p.stdout.on('data', (data) => {
                    allData.push(data);
                    if (data.toString('utf-8').indexOf('No cameras available') > -1) {
                        p.kill('SIGINT');
                        resolve(Buffer.concat(allData).toString('utf-8'));
                    }
                });
                p.stderr.on('data', (data) => {
                    allData.push(data);
                    if (data.toString('utf-8').indexOf('No cameras available') > -1) {
                        p.kill('SIGINT');
                        resolve(Buffer.concat(allData).toString('utf-8'));
                    }
                });
                p.on('error', reject);
                p.on('close', (code) => {
                    if (code === 0 || opts.ignoreErrors === true) {
                        resolve(Buffer.concat(allData).toString('utf-8'));
                    }
                    else {
                        reject('Error code was not 0: ' + Buffer.concat(allData).toString('utf-8'));
                    }
                });
            });
        }
        else {
            let command = 'gst-launch-1.0';
            let args = ['nvarguscamerasrc'];
            let opts = { ignoreErrors: true };
            gstLaunchRet = await this._spawnHelper(command, args, opts);
        }
        let lines = gstLaunchRet.split('\n').filter(x => !!x).map(x => x.trim());
        lines = lines.filter(x => x.startsWith('GST_ARGUS:'));
        if (this._verbose) {
            console.log(PREFIX, 'gst-launch-1.0 nvarguscamerasrc options', lines.join('\n'));
        }
        for (let l of lines) {
            let m = l.match(/^GST_ARGUS: (\d+)(?:\s*)x(?:\s*)(\d+).*?=(?:\s*)([\d,\.]+)(?:\s*)fps/);
            if (!m) {
                continue;
            }
            let cap = {
                framerate: Number(m[3].replace(',', '.')),
                height: Number(m[2]),
                width: Number(m[1]),
                type: 'nvarguscamerasrc'
            };
            if (!isNaN(cap.width) && !isNaN(cap.height) && !isNaN(cap.framerate)) {
                caps.push(cap);
            }
        }
        if (caps.length > 0) {
            let d = {
                caps: caps,
                deviceClass: '',
                id: 'nvarguscamerasrc',
                inCapMode: false,
                name: 'CSI camera',
                rawCaps: [],
                videoSource: 'nvarguscamerasrc',
            };
            return [d];
        }
        else {
            return [];
        }
    }
    async listPylonsrcDevices() {
        const hasPlugin = await this.hasGstPlugin('pylonsrc');
        if (!hasPlugin) {
            return [];
        }
        let caps = [];
        let gstInspectRet;
        {
            let command = 'gst-inspect-1.0';
            let args = ['pylonsrc'];
            let opts = { ignoreErrors: true };
            // not overridden spawn helper?
            if (this._spawnHelper === spawn_helper_1.spawnHelper) {
                gstInspectRet = await new Promise((resolve, reject) => {
                    const p = (0, child_process_1.spawn)(command, args, { env: process.env });
                    let allData = [];
                    p.stdout.on('data', (data) => {
                        allData.push(data);
                        if (data.toString('utf-8').indexOf('No devices found') > -1) {
                            p.kill('SIGINT');
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                    p.stderr.on('data', (data) => {
                        allData.push(data);
                        if (data.toString('utf-8').indexOf('No devices found') > -1) {
                            p.kill('SIGINT');
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                    p.on('error', reject);
                    p.on('close', (code) => {
                        if (code === 0 || opts.ignoreErrors === true) {
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                        else {
                            reject('Error code was not 0: ' + Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                });
            }
            else {
                gstInspectRet = await this._spawnHelper(command, args, opts);
            }
        }
        let gstInspectLines = gstInspectRet.split('\n');
        let inRoiWidth = false;
        let inRoiHeight = false;
        let currProp = [];
        let expectedIndent = 0;
        let defaultWidth = 1920;
        let defaultHeight = 1080;
        // find AutoFunctionROIWidth-ROI1 and AutoFunctionROIHeight-ROI1
        for (const line of gstInspectLines) {
            let currIndent = line.split(/\w+/)[0].length;
            if (line.indexOf('AutoFunctionROIWidth-ROI1') > -1) {
                inRoiWidth = true;
                expectedIndent = currIndent;
                currProp = [line];
                continue;
            }
            else if (line.indexOf('AutoFunctionROIHeight-ROI1') > -1) {
                inRoiHeight = true;
                expectedIndent = currIndent;
                currProp = [line];
                continue;
            }
            if (inRoiWidth) {
                if (currIndent === expectedIndent) {
                    let roiWidth = currProp.join('\n');
                    if (this._verbose) {
                        console.log(PREFIX, 'AutoFunctionROIWidth-ROI1:\n', roiWidth);
                    }
                    let m = roiWidth.match(/Default\: (\d+)/);
                    if (m) {
                        defaultWidth = Number(m[1]);
                    }
                    inRoiWidth = false;
                }
                else {
                    currProp.push(line);
                }
            }
            if (inRoiHeight) {
                if (currIndent === expectedIndent) {
                    let roiHeight = currProp.join('\n');
                    if (this._verbose) {
                        console.log(PREFIX, 'AutoFunctionROIHeight-ROI1:\n', roiHeight);
                    }
                    let m = roiHeight.match(/Default\: (\d+)/);
                    if (m) {
                        defaultHeight = Number(m[1]);
                    }
                    inRoiHeight = false;
                }
                else {
                    currProp.push(line);
                }
            }
        }
        // console.log('gstInspectRes', gstInspectRet);
        let gstLaunchRet;
        {
            let command = 'gst-launch-1.0';
            let args = ['pylonsrc'];
            let opts = { ignoreErrors: true };
            // not overridden spawn helper?
            if (this._spawnHelper === spawn_helper_1.spawnHelper) {
                gstLaunchRet = await new Promise((resolve, reject) => {
                    const p = (0, child_process_1.spawn)(command, args, { env: process.env });
                    let allData = [];
                    p.stdout.on('data', (data) => {
                        allData.push(data);
                        if (data.toString('utf-8').indexOf('No devices found') > -1) {
                            p.kill('SIGINT');
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                    p.stderr.on('data', (data) => {
                        allData.push(data);
                        if (data.toString('utf-8').indexOf('No devices found') > -1) {
                            p.kill('SIGINT');
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                    p.on('error', reject);
                    p.on('close', (code) => {
                        if (code === 0 || opts.ignoreErrors === true) {
                            resolve(Buffer.concat(allData).toString('utf-8'));
                        }
                        else {
                            reject('Error code was not 0: ' + Buffer.concat(allData).toString('utf-8'));
                        }
                    });
                });
            }
            else {
                gstLaunchRet = await this._spawnHelper(command, args, opts);
            }
        }
        let lines = gstLaunchRet.split('\n').filter(x => !!x).map(x => x.trim());
        if (this._verbose) {
            console.log(PREFIX, 'gst-launch-1.0 pylonsrc options', lines.join('\n'));
        }
        for (let l of lines) {
            let m = l.match(/New clock: GstSystemClock/);
            if (!m) {
                continue;
            }
            // fps,height and width won't be used
            let cap = {
                framerate: 60,
                height: defaultHeight,
                width: defaultWidth,
                type: 'pylonsrc'
            };
            caps.push(cap);
        }
        if (caps.length > 0) {
            let d = {
                caps: caps,
                deviceClass: '',
                id: 'pylonsrc',
                inCapMode: false,
                name: 'Basler camera',
                rawCaps: [],
                videoSource: 'pylonsrc',
            };
            return [d];
        }
        else {
            return [];
        }
    }
    getLastOptions() {
        return this._lastOptions;
    }
    async exists(path) {
        let exists = false;
        try {
            await util_1.default.promisify(fs_1.default.stat)(path);
            exists = true;
        }
        catch (ex) {
            /* noop */
        }
        return exists;
    }
    async timeoutCallback() {
        try {
            this._isRestarting = true;
            if (this._verbose) {
                console.log(PREFIX, 'No images received for 2 seconds, restarting...');
            }
            await this.stop();
            if (this._verbose) {
                console.log(PREFIX, 'Stopped');
            }
            await new Promise((resolve) => setTimeout(resolve, 500));
            if (this._lastOptions) {
                if (this._verbose) {
                    console.log(PREFIX, 'Starting gstreamer...');
                }
                await this.start(this._lastOptions);
                if (this._verbose) {
                    console.log(PREFIX, 'Restart completed');
                }
            }
            else {
                this.emit('error', 'gstreamer process went stale');
            }
        }
        catch (ex2) {
            let ex = ex2;
            this.emit('error', 'gstreamer failed to restart: ' + (ex.message || ex.toString()));
        }
        finally {
            this._isRestarting = false;
        }
    }
}
exports.GStreamer = GStreamer;
//# sourceMappingURL=gstreamer.js.map