"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseUploaderInfo = exports.calculateAllStructuredLabels = exports.verifyStructuredLabels = exports.verifyStructuredLabelsString = exports.parseStructuredLabelsString = exports.validateStructuredLabelsFile = exports.parseStructuredLabelsFile = exports.verifyBoundingBoxes = exports.validateBoundingBoxLabelsFile = exports.parseBoundingBoxLabels = void 0;
function parseBoundingBoxLabels(jsonFile) {
    const data = JSON.parse(jsonFile);
    return validateBoundingBoxLabelsFile(data);
}
exports.parseBoundingBoxLabels = parseBoundingBoxLabels;
function validateBoundingBoxLabelsFile(data) {
    if (data.version !== 1) {
        throw new Error('Invalid version');
    }
    if (data.type !== 'bounding-box-labels') {
        throw new Error('Invalid type');
    }
    if (typeof data.boundingBoxes !== 'object') {
        throw new Error('boundingBoxes is not an object');
    }
    return data;
}
exports.validateBoundingBoxLabelsFile = validateBoundingBoxLabelsFile;
function verifyBoundingBoxes(boundingBoxes) {
    if (!Array.isArray(boundingBoxes)) {
        throw new Error('boundingBoxes should be an array');
    }
    for (let b of boundingBoxes) {
        if (typeof b.label !== 'string') {
            throw new Error('label is not a string in ' + JSON.stringify(b));
        }
        if (typeof b.x !== 'number') {
            throw new Error('x is not a number in ' + JSON.stringify(b));
        }
        if (typeof b.y !== 'number') {
            throw new Error('y is not a number in ' + JSON.stringify(b));
        }
        if (typeof b.width !== 'number') {
            throw new Error('width is not a number in ' + JSON.stringify(b));
        }
        if (typeof b.height !== 'number') {
            throw new Error('height is not a number in ' + JSON.stringify(b));
        }
        if (b.width <= 0) {
            throw new Error('width should be >0 in ' + JSON.stringify(b));
        }
        if (b.height <= 0) {
            throw new Error('height should be >0 in ' + JSON.stringify(b));
        }
        // we post this from the UI, we should not save it in the DB
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        delete b.addedByHand;
    }
}
exports.verifyBoundingBoxes = verifyBoundingBoxes;
function parseStructuredLabelsFile(jsonFile) {
    const data = JSON.parse(jsonFile);
    return validateStructuredLabelsFile(data);
}
exports.parseStructuredLabelsFile = parseStructuredLabelsFile;
function validateStructuredLabelsFile(data) {
    if (data.version !== 1) {
        throw new Error('Invalid version');
    }
    if (data.type !== 'structured-labels') {
        throw new Error('Invalid type, expected "structured-labels" but was "' + data.type + '"');
    }
    if (typeof data.structuredLabels !== 'object') {
        throw new Error('structuredLabels is not an object');
    }
    // inner structure will be validated by ingestion
    return data;
}
exports.validateStructuredLabelsFile = validateStructuredLabelsFile;
function parseStructuredLabelsString(json, source) {
    let labels;
    try {
        labels = JSON.parse(json);
    }
    catch (ex) {
        throw new Error(source + ' is not valid json');
    }
    return labels;
}
exports.parseStructuredLabelsString = parseStructuredLabelsString;
function verifyStructuredLabelsString(json, valuesCount, source) {
    const labels = parseStructuredLabelsString(json, source);
    return verifyStructuredLabels(labels, valuesCount, source);
}
exports.verifyStructuredLabelsString = verifyStructuredLabelsString;
function verifyStructuredLabels(labels, valuesCount, source) {
    if (!Array.isArray(labels)) {
        throw new Error(`${source} is not a valid array`);
    }
    for (let ix = 0; ix < labels.length; ix++) {
        const l = labels[ix];
        if (typeof l.label !== 'string') {
            throw new Error(`${source} (index ${ix}) "label" is required and should be string. Type is ${typeof l.label}`);
        }
        if (typeof l.startIndex !== 'number') {
            throw new Error(`${source} (index ${ix}) "startIndex" is required and should be numeric`);
        }
        if (typeof l.endIndex !== 'number') {
            throw new Error(`${source} (index ${ix}) "endIndex" is required and should be numeric`);
        }
        if (l.startIndex !== Math.floor(l.startIndex)) {
            throw new Error(`${source} (index ${ix}) "startIndex" should be an integer`);
        }
        if (l.endIndex !== Math.floor(l.endIndex)) {
            throw new Error(`${source} (index ${ix}) "endIndex" should be an integer`);
        }
        if (l.startIndex < 0) {
            throw new Error(`${source} (index ${ix}) "startIndex" should be >= 0`);
        }
        if (l.startIndex > l.endIndex) {
            throw new Error(`${source} (index ${ix}) "endIndex" should be after "startIndex"`);
        }
        if (l.startIndex >= valuesCount) {
            throw new Error(`${source} (index ${ix}) "startIndex" is out of bounds, max. index is ${valuesCount - 1}`);
        }
        if (l.endIndex >= valuesCount) {
            throw new Error(`${source} (index ${ix}) "endIndex" is out of bounds, max. index is ${valuesCount - 1}`);
        }
    }
    // this validates that we have labels for every index
    let allLabels = calculateAllStructuredLabels({
        structuredLabels: labels,
        valuesCount: valuesCount,
    });
    // based on the allLabels, reconstruct the structured labels
    // into new boxes (this'll have no overlap, and will merge labels)
    let currStartIx = 0;
    let currLabel = allLabels[0];
    let ret = [];
    for (let ix = 1; ix < valuesCount; ix++) {
        if (allLabels[ix] !== currLabel) {
            ret.push({
                startIndex: currStartIx,
                endIndex: ix - 1,
                label: currLabel,
            });
            currStartIx = ix;
        }
        currLabel = allLabels[ix];
    }
    ret.push({
        startIndex: currStartIx,
        endIndex: valuesCount - 1,
        label: currLabel,
    });
    return ret;
}
exports.verifyStructuredLabels = verifyStructuredLabels;
/**
 * This returns a new string array of length `sample.valuesCount`, and each
 * value of the array contains the label of the sample at that specific index.
 * If you have a sample that has structured labels you can use this to easily
 * find what the expected label was for the sample at a certain index.
 * If the sample has no structured labels it'll return undefined.
 * @param sample The sample
 * @returns A string[] with labels (or undefined if no structured labels)
 */
function calculateAllStructuredLabels(sample) {
    let allStructuredLabels = Array.from({ length: sample.valuesCount }).fill(undefined);
    for (let structuredLabel of sample.structuredLabels) {
        // endIx is inclusive
        for (let ix = structuredLabel.startIndex; ix <= structuredLabel.endIndex; ix++) {
            if (ix > sample.valuesCount + 1) {
                throw new Error(`structuredLabels: index ${ix} is out of bounds. ` +
                    `Max index is ${sample.valuesCount + 1}.`);
            }
            allStructuredLabels[ix] = structuredLabel.label;
        }
    }
    let undefinedIx = allStructuredLabels === null || allStructuredLabels === void 0 ? void 0 : allStructuredLabels.findIndex(l => typeof l === 'undefined');
    if (undefinedIx !== -1) {
        throw new Error(`structuredLabels: index ${undefinedIx} does not have a label. ` +
            `Currently we require structured labels for the complete sample, there cannot be any gaps in the labels.`);
    }
    return allStructuredLabels;
}
exports.calculateAllStructuredLabels = calculateAllStructuredLabels;
function parseUploaderInfo(jsonFile) {
    let data = JSON.parse(jsonFile);
    if (data.version !== 1) {
        throw new Error('Invalid version');
    }
    if (!Array.isArray(data.files)) {
        throw new Error('files is not an array');
    }
    return data;
}
exports.parseUploaderInfo = parseUploaderInfo;
//# sourceMappingURL=bounding-box-file-types.js.map